# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test helpers for RPC invocation tests."""

import datetime
import threading

import grpc
from grpc.framework.foundation import logging_pool

from tests.unit import test_common
from tests.unit import thread_pool
from tests.unit.framework.common import test_constants
from tests.unit.framework.common import test_control

_SERIALIZE_REQUEST = lambda bytestring: bytestring * 2
_DESERIALIZE_REQUEST = lambda bytestring: bytestring[len(bytestring) // 2:]
_SERIALIZE_RESPONSE = lambda bytestring: bytestring * 3
_DESERIALIZE_RESPONSE = lambda bytestring: bytestring[:len(bytestring) // 3]

_UNARY_UNARY = '/test/UnaryUnary'
_UNARY_STREAM = '/test/UnaryStream'
_UNARY_STREAM_NON_BLOCKING = '/test/UnaryStreamNonBlocking'
_STREAM_UNARY = '/test/StreamUnary'
_STREAM_STREAM = '/test/StreamStream'
_STREAM_STREAM_NON_BLOCKING = '/test/StreamStreamNonBlocking'

TIMEOUT_SHORT = datetime.timedelta(seconds=4).total_seconds()


class Callback(object):

    def __init__(self):
        self._condition = threading.Condition()
        self._value = None
        self._called = False

    def __call__(self, value):
        with self._condition:
            self._value = value
            self._called = True
            self._condition.notify_all()

    def value(self):
        with self._condition:
            while not self._called:
                self._condition.wait()
            return self._value


class _Handler(object):

    def __init__(self, control, thread_pool):
        self._control = control
        self._thread_pool = thread_pool
        non_blocking_functions = (self.handle_unary_stream_non_blocking,
                                  self.handle_stream_stream_non_blocking)
        for non_blocking_function in non_blocking_functions:
            non_blocking_function.__func__.experimental_non_blocking = True
            non_blocking_function.__func__.experimental_thread_pool = self._thread_pool

    def handle_unary_unary(self, request, servicer_context):
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(((
                'testkey',
                'testvalue',
            ),))
            # TODO(https://github.com/grpc/grpc/issues/8483): test the values
            # returned by these methods rather than only "smoke" testing that
            # the return after having been called.
            servicer_context.is_active()
            servicer_context.time_remaining()
        return request

    def handle_unary_stream(self, request, servicer_context):
        for _ in range(test_constants.STREAM_LENGTH):
            self._control.control()
            yield request
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(((
                'testkey',
                'testvalue',
            ),))

    def handle_unary_stream_non_blocking(self, request, servicer_context,
                                         on_next):
        for _ in range(test_constants.STREAM_LENGTH):
            self._control.control()
            on_next(request)
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(((
                'testkey',
                'testvalue',
            ),))
        on_next(None)

    def handle_stream_unary(self, request_iterator, servicer_context):
        if servicer_context is not None:
            servicer_context.invocation_metadata()
        self._control.control()
        response_elements = []
        for request in request_iterator:
            self._control.control()
            response_elements.append(request)
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(((
                'testkey',
                'testvalue',
            ),))
        return b''.join(response_elements)

    def handle_stream_stream(self, request_iterator, servicer_context):
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(((
                'testkey',
                'testvalue',
            ),))
        for request in request_iterator:
            self._control.control()
            yield request
        self._control.control()

    def handle_stream_stream_non_blocking(self, request_iterator,
                                          servicer_context, on_next):
        self._control.control()
        if servicer_context is not None:
            servicer_context.set_trailing_metadata(((
                'testkey',
                'testvalue',
            ),))
        for request in request_iterator:
            self._control.control()
            on_next(request)
        self._control.control()
        on_next(None)


class _MethodHandler(grpc.RpcMethodHandler):

    def __init__(self, request_streaming, response_streaming,
                 request_deserializer, response_serializer, unary_unary,
                 unary_stream, stream_unary, stream_stream):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = request_deserializer
        self.response_serializer = response_serializer
        self.unary_unary = unary_unary
        self.unary_stream = unary_stream
        self.stream_unary = stream_unary
        self.stream_stream = stream_stream


class _GenericHandler(grpc.GenericRpcHandler):

    def __init__(self, handler):
        self._handler = handler

    def service(self, handler_call_details):
        if handler_call_details.method == _UNARY_UNARY:
            return _MethodHandler(False, False, None, None,
                                  self._handler.handle_unary_unary, None, None,
                                  None)
        elif handler_call_details.method == _UNARY_STREAM:
            return _MethodHandler(False, True, _DESERIALIZE_REQUEST,
                                  _SERIALIZE_RESPONSE, None,
                                  self._handler.handle_unary_stream, None, None)
        elif handler_call_details.method == _UNARY_STREAM_NON_BLOCKING:
            return _MethodHandler(
                False, True, _DESERIALIZE_REQUEST, _SERIALIZE_RESPONSE, None,
                self._handler.handle_unary_stream_non_blocking, None, None)
        elif handler_call_details.method == _STREAM_UNARY:
            return _MethodHandler(True, False, _DESERIALIZE_REQUEST,
                                  _SERIALIZE_RESPONSE, None, None,
                                  self._handler.handle_stream_unary, None)
        elif handler_call_details.method == _STREAM_STREAM:
            return _MethodHandler(True, True, None, None, None, None, None,
                                  self._handler.handle_stream_stream)
        elif handler_call_details.method == _STREAM_STREAM_NON_BLOCKING:
            return _MethodHandler(
                True, True, None, None, None, None, None,
                self._handler.handle_stream_stream_non_blocking)
        else:
            return None


def unary_unary_multi_callable(channel):
    return channel.unary_unary(_UNARY_UNARY)


def unary_stream_multi_callable(channel):
    return channel.unary_stream(_UNARY_STREAM,
                                request_serializer=_SERIALIZE_REQUEST,
                                response_deserializer=_DESERIALIZE_RESPONSE)


def unary_stream_non_blocking_multi_callable(channel):
    return channel.unary_stream(_UNARY_STREAM_NON_BLOCKING,
                                request_serializer=_SERIALIZE_REQUEST,
                                response_deserializer=_DESERIALIZE_RESPONSE)


def stream_unary_multi_callable(channel):
    return channel.stream_unary(_STREAM_UNARY,
                                request_serializer=_SERIALIZE_REQUEST,
                                response_deserializer=_DESERIALIZE_RESPONSE)


def stream_stream_multi_callable(channel):
    return channel.stream_stream(_STREAM_STREAM)


def stream_stream_non_blocking_multi_callable(channel):
    return channel.stream_stream(_STREAM_STREAM_NON_BLOCKING)


class BaseRPCTest(object):

    def setUp(self):
        self._control = test_control.PauseFailControl()
        self._thread_pool = thread_pool.RecordingThreadPool(max_workers=None)
        self._handler = _Handler(self._control, self._thread_pool)

        self._server = test_common.test_server()
        port = self._server.add_insecure_port('[::]:0')
        self._server.add_generic_rpc_handlers((_GenericHandler(self._handler),))
        self._server.start()

        self._channel = grpc.insecure_channel('localhost:%d' % port)

    def tearDown(self):
        self._server.stop(None)
        self._channel.close()

    def _consume_one_stream_response_unary_request(self, multi_callable):
        request = b'\x57\x38'

        response_iterator = multi_callable(
            request,
            metadata=(('test', 'ConsumingOneStreamResponseUnaryRequest'),))
        next(response_iterator)

    def _consume_some_but_not_all_stream_responses_unary_request(
            self, multi_callable):
        request = b'\x57\x38'

        response_iterator = multi_callable(
            request,
            metadata=(('test',
                       'ConsumingSomeButNotAllStreamResponsesUnaryRequest'),))
        for _ in range(test_constants.STREAM_LENGTH // 2):
            next(response_iterator)

    def _consume_some_but_not_all_stream_responses_stream_request(
            self, multi_callable):
        requests = tuple(
            b'\x67\x88' for _ in range(test_constants.STREAM_LENGTH))
        request_iterator = iter(requests)

        response_iterator = multi_callable(
            request_iterator,
            metadata=(('test',
                       'ConsumingSomeButNotAllStreamResponsesStreamRequest'),))
        for _ in range(test_constants.STREAM_LENGTH // 2):
            next(response_iterator)

    def _consume_too_many_stream_responses_stream_request(self, multi_callable):
        requests = tuple(
            b'\x67\x88' for _ in range(test_constants.STREAM_LENGTH))
        request_iterator = iter(requests)

        response_iterator = multi_callable(
            request_iterator,
            metadata=(('test',
                       'ConsumingTooManyStreamResponsesStreamRequest'),))
        for _ in range(test_constants.STREAM_LENGTH):
            next(response_iterator)
        for _ in range(test_constants.STREAM_LENGTH):
            with self.assertRaises(StopIteration):
                next(response_iterator)

        self.assertIsNotNone(response_iterator.initial_metadata())
        self.assertIs(grpc.StatusCode.OK, response_iterator.code())
        self.assertIsNotNone(response_iterator.details())
        self.assertIsNotNone(response_iterator.trailing_metadata())

    def _cancelled_unary_request_stream_response(self, multi_callable):
        request = b'\x07\x19'

        with self._control.pause():
            response_iterator = multi_callable(
                request,
                metadata=(('test', 'CancelledUnaryRequestStreamResponse'),))
            self._control.block_until_paused()
            response_iterator.cancel()

        with self.assertRaises(grpc.RpcError) as exception_context:
            next(response_iterator)
        self.assertIs(grpc.StatusCode.CANCELLED,
                      exception_context.exception.code())
        self.assertIsNotNone(response_iterator.initial_metadata())
        self.assertIs(grpc.StatusCode.CANCELLED, response_iterator.code())
        self.assertIsNotNone(response_iterator.details())
        self.assertIsNotNone(response_iterator.trailing_metadata())

    def _cancelled_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b'\x07\x08' for _ in range(test_constants.STREAM_LENGTH))
        request_iterator = iter(requests)

        with self._control.pause():
            response_iterator = multi_callable(
                request_iterator,
                metadata=(('test', 'CancelledStreamRequestStreamResponse'),))
            response_iterator.cancel()

        with self.assertRaises(grpc.RpcError):
            next(response_iterator)
        self.assertIsNotNone(response_iterator.initial_metadata())
        self.assertIs(grpc.StatusCode.CANCELLED, response_iterator.code())
        self.assertIsNotNone(response_iterator.details())
        self.assertIsNotNone(response_iterator.trailing_metadata())

    def _expired_unary_request_stream_response(self, multi_callable):
        request = b'\x07\x19'

        with self._control.pause():
            with self.assertRaises(grpc.RpcError) as exception_context:
                response_iterator = multi_callable(
                    request,
                    timeout=test_constants.SHORT_TIMEOUT,
                    metadata=(('test', 'ExpiredUnaryRequestStreamResponse'),))
                next(response_iterator)

        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED,
                      exception_context.exception.code())
        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED,
                      response_iterator.code())

    def _expired_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b'\x67\x18' for _ in range(test_constants.STREAM_LENGTH))
        request_iterator = iter(requests)

        with self._control.pause():
            with self.assertRaises(grpc.RpcError) as exception_context:
                response_iterator = multi_callable(
                    request_iterator,
                    timeout=test_constants.SHORT_TIMEOUT,
                    metadata=(('test', 'ExpiredStreamRequestStreamResponse'),))
                next(response_iterator)

        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED,
                      exception_context.exception.code())
        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED,
                      response_iterator.code())

    def _failed_unary_request_stream_response(self, multi_callable):
        request = b'\x37\x17'

        with self.assertRaises(grpc.RpcError) as exception_context:
            with self._control.fail():
                response_iterator = multi_callable(
                    request,
                    metadata=(('test', 'FailedUnaryRequestStreamResponse'),))
                next(response_iterator)

        self.assertIs(grpc.StatusCode.UNKNOWN,
                      exception_context.exception.code())

    def _failed_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b'\x67\x88' for _ in range(test_constants.STREAM_LENGTH))
        request_iterator = iter(requests)

        with self._control.fail():
            with self.assertRaises(grpc.RpcError) as exception_context:
                response_iterator = multi_callable(
                    request_iterator,
                    metadata=(('test', 'FailedStreamRequestStreamResponse'),))
                tuple(response_iterator)

        self.assertIs(grpc.StatusCode.UNKNOWN,
                      exception_context.exception.code())
        self.assertIs(grpc.StatusCode.UNKNOWN, response_iterator.code())

    def _ignored_unary_stream_request_future_unary_response(
            self, multi_callable):
        request = b'\x37\x17'

        multi_callable(request,
                       metadata=(('test',
                                  'IgnoredUnaryRequestStreamResponse'),))

    def _ignored_stream_request_stream_response(self, multi_callable):
        requests = tuple(
            b'\x67\x88' for _ in range(test_constants.STREAM_LENGTH))
        request_iterator = iter(requests)

        multi_callable(request_iterator,
                       metadata=(('test',
                                  'IgnoredStreamRequestStreamResponse'),))



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests 'utf-8' encoded error message."""

import unittest
import weakref

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_UNICODE_ERROR_MESSAGES = [
    b'\xe2\x80\x9d'.decode('utf-8'),
    b'abc\x80\xd0\xaf'.decode('latin-1'),
    b'\xc3\xa9'.decode('utf-8'),
]

_REQUEST = b'\x00\x00\x00'
_RESPONSE = b'\x00\x00\x00'

_UNARY_UNARY = '/test/UnaryUnary'


class _MethodHandler(grpc.RpcMethodHandler):

    def __init__(self, request_streaming=None, response_streaming=None):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None

    def unary_unary(self, request, servicer_context):
        servicer_context.set_code(grpc.StatusCode.UNKNOWN)
        servicer_context.set_details(request.decode('utf-8'))
        return _RESPONSE


class _GenericHandler(grpc.GenericRpcHandler):

    def __init__(self, test):
        self._test = test

    def service(self, handler_call_details):
        return _MethodHandler()


class ErrorMessageEncodingTest(unittest.TestCase):

    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers(
            (_GenericHandler(weakref.proxy(self)),))
        port = self._server.add_insecure_port('[::]:0')
        self._server.start()
        self._channel = grpc.insecure_channel('localhost:%d' % port)

    def tearDown(self):
        self._server.stop(0)
        self._channel.close()

    def testMessageEncoding(self):
        for message in _UNICODE_ERROR_MESSAGES:
            multi_callable = self._channel.unary_unary(_UNARY_UNARY)
            with self.assertRaises(grpc.RpcError) as cm:
                multi_callable(message.encode('utf-8'))

            self.assertEqual(cm.exception.code(), grpc.StatusCode.UNKNOWN)
            self.assertEqual(cm.exception.details(), message)


if __name__ == '__main__':
    unittest.main(verbosity=2)



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import datetime
import logging

from absl import flags
from absl.testing import absltest
import grpc

from framework import xds_k8s_testcase

logger = logging.getLogger(__name__)
flags.adopt_module_key_flags(xds_k8s_testcase)

# Type aliases
_XdsTestServer = xds_k8s_testcase.XdsTestServer
_XdsTestClient = xds_k8s_testcase.XdsTestClient

_EXPECTED_STATUS = grpc.StatusCode.DATA_LOSS


class CustomLbTest(xds_k8s_testcase.RegularXdsKubernetesTestCase):

    def test_custom_lb_config(self):
        with self.subTest('0_create_health_check'):
            self.td.create_health_check()

        # Configures a custom, test LB on the client to instruct the servers
        # to always respond with a specific error code.
        with self.subTest('1_create_backend_service'):
            self.td.create_backend_service(locality_lb_policies=[{
                'customPolicy': {
                    'name':
                        'test.RpcBehaviorLoadBalancer',
                    'data':
                        f'{{ "rpcBehavior": "error-code-{_EXPECTED_STATUS.value[0]}" }}'
                }
            }])

        with self.subTest('2_create_url_map'):
            self.td.create_url_map(self.server_xds_host, self.server_xds_port)

        with self.subTest('3_create_target_proxy'):
            self.td.create_target_proxy()

        with self.subTest('4_create_forwarding_rule'):
            self.td.create_forwarding_rule(self.server_xds_port)

        with self.subTest('5_start_test_server'):
            test_server: _XdsTestServer = self.startTestServers()[0]

        with self.subTest('6_add_server_backends_to_backend_service'):
            self.setupServerBackends()

        with self.subTest('7_start_test_client'):
            test_client: _XdsTestClient = self.startTestClient(test_server)

        with self.subTest('8_test_client_xds_config_exists'):
            self.assertXdsConfigExists(test_client)

        # Verify status codes from the servers have the configured one.
        with self.subTest('9_test_server_returned_configured_status_code'):
            self.assertRpcStatusCodes(test_client,
                                      status_code=_EXPECTED_STATUS,
                                      duration=datetime.timedelta(seconds=10),
                                      method='UNARY_CALL')


if __name__ == '__main__':
    absltest.main(failfast=True)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import logging
import os
import socket
import subprocess
import sys
import tempfile
import time

import six.moves.urllib.request as request

sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import jobset

# must be synchronized with test/core/util/port_server_client.h
_PORT_SERVER_PORT = 32766


def start_port_server():
    # check if a compatible port server is running
    # if incompatible (version mismatch) ==> start a new one
    # if not running ==> start a new one
    # otherwise, leave it up
    try:
        version = int(
            request.urlopen('http://localhost:%d/version_number' %
                            _PORT_SERVER_PORT).read())
        logging.info('detected port server running version %d', version)
        running = True
    except Exception as e:
        logging.exception('failed to detect port server')
        running = False
    if running:
        current_version = int(
            subprocess.check_output([
                sys.executable,  # use the same python binary as this process
                os.path.abspath('tools/run_tests/python_utils/port_server.py'),
                'dump_version'
            ]).decode())
        logging.info('my port server is version %d', current_version)
        running = (version >= current_version)
        if not running:
            logging.info('port_server version mismatch: killing the old one')
            request.urlopen('http://localhost:%d/quitquitquit' %
                            _PORT_SERVER_PORT).read()
            time.sleep(1)
    if not running:
        fd, logfile = tempfile.mkstemp()
        os.close(fd)
        logging.info('starting port_server, with log file %s', logfile)
        args = [
            sys.executable,
            os.path.abspath('tools/run_tests/python_utils/port_server.py'),
            '-p',
            '%d' % _PORT_SERVER_PORT, '-l', logfile
        ]
        env = dict(os.environ)
        env['BUILD_ID'] = 'pleaseDontKillMeJenkins'
        if jobset.platform_string() == 'windows':
            # Working directory of port server needs to be outside of Jenkins
            # workspace to prevent file lock issues.
            tempdir = tempfile.mkdtemp()
            if sys.version_info.major == 2:
                creationflags = 0x00000008  # detached process
            else:
                creationflags = 0  # DETACHED_PROCESS doesn't seem to work with python3
            port_server = subprocess.Popen(args,
                                           env=env,
                                           cwd=tempdir,
                                           creationflags=creationflags,
                                           close_fds=True)
        else:
            port_server = subprocess.Popen(args,
                                           env=env,
                                           preexec_fn=os.setsid,
                                           close_fds=True)
        time.sleep(1)
        # ensure port server is up
        waits = 0
        while True:
            if waits > 10:
                logging.warning(
                    'killing port server due to excessive start up waits')
                port_server.kill()
            if port_server.poll() is not None:
                logging.error('port_server failed to start')
                # try one final time: maybe another build managed to start one
                time.sleep(1)
                try:
                    request.urlopen('http://localhost:%d/get' %
                                    _PORT_SERVER_PORT).read()
                    logging.info(
                        'last ditch attempt to contact port server succeeded')
                    break
                except:
                    logging.exception(
                        'final attempt to contact port server failed')
                    port_log = open(logfile, 'r').read()
                    print(port_log)
                    sys.exit(1)
            try:
                port_server_url = 'http://localhost:%d/get' % _PORT_SERVER_PORT
                request.urlopen(port_server_url).read()
                logging.info('port server is up and ready')
                break
            except socket.timeout:
                logging.exception('while waiting for port_server')
                time.sleep(1)
                waits += 1
            except IOError:
                logging.exception('while waiting for port_server')
                time.sleep(1)
                waits += 1
            except:
                logging.exception(
                    'error while contacting port server at "%s".'
                    'Will try killing it.', port_server_url)
                port_server.kill()
                raise



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the gRPC client-side retry example."""

import json
import logging

import grpc

helloworld_pb2, helloworld_pb2_grpc = grpc.protos_and_services(
    "helloworld.proto")


def run():
    # The ServiceConfig proto definition can be found:
    # https://github.com/grpc/grpc-proto/blob/ec886024c2f7b7f597ba89d5b7d60c3f94627b17/grpc/service_config/service_config.proto#L377
    service_config_json = json.dumps({
        "methodConfig": [{
            # To apply retry to all methods, put [{}] in the "name" field
            "name": [{
                "service": "helloworld.Greeter",
                "method": "SayHello"
            }],
            "retryPolicy": {
                "maxAttempts": 5,
                "initialBackoff": "0.1s",
                "maxBackoff": "1s",
                "backoffMultiplier": 2,
                "retryableStatusCodes": ["UNAVAILABLE"],
            },
        }]
    })
    options = []
    # NOTE: the retry feature will be enabled by default >=v1.40.0
    options.append(("grpc.enable_retries", 1))
    options.append(("grpc.service_config", service_config_json))
    with grpc.insecure_channel('localhost:50051', options=options) as channel:
        stub = helloworld_pb2_grpc.GreeterStub(channel)
        response = stub.SayHello(helloworld_pb2.HelloRequest(name='you'))
    print("Greeter client received: " + response.message)


if __name__ == '__main__':
    logging.basicConfig()
    run()



#!/usr/bin/env python3

# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import errno
import filecmp
import glob
import os
import os.path
import shutil
import subprocess
import sys
import traceback
import uuid

DEPS_FILE_CONTENT = """
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# AUTO-GENERATED BY make_grpcio_tools.py!
CC_FILES={cc_files}
PROTO_FILES={proto_files}

CC_INCLUDE={cc_include}
PROTO_INCLUDE={proto_include}

{commit_hash}
"""

COMMIT_HASH_PREFIX = 'PROTOBUF_SUBMODULE_VERSION="'
COMMIT_HASH_SUFFIX = '"'

# Bazel query result prefix for expected source files in protobuf.
PROTOBUF_CC_PREFIX = '//:src/'
PROTOBUF_PROTO_PREFIX = '//:src/'

GRPC_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', '..', '..'))

GRPC_PYTHON_ROOT = os.path.join(GRPC_ROOT, 'tools', 'distrib', 'python',
                                'grpcio_tools')

GRPC_PYTHON_PROTOBUF_RELATIVE_ROOT = os.path.join('third_party', 'protobuf',
                                                  'src')
GRPC_PROTOBUF = os.path.join(GRPC_ROOT, GRPC_PYTHON_PROTOBUF_RELATIVE_ROOT)
GRPC_PROTOBUF_SUBMODULE_ROOT = os.path.join(GRPC_ROOT, 'third_party',
                                            'protobuf')
GRPC_PROTOC_PLUGINS = os.path.join(GRPC_ROOT, 'src', 'compiler')
GRPC_PYTHON_PROTOBUF = os.path.join(GRPC_PYTHON_ROOT, 'third_party', 'protobuf',
                                    'src')
GRPC_PYTHON_PROTOC_PLUGINS = os.path.join(GRPC_PYTHON_ROOT, 'grpc_root', 'src',
                                          'compiler')
GRPC_PYTHON_PROTOC_LIB_DEPS = os.path.join(GRPC_PYTHON_ROOT,
                                           'protoc_lib_deps.py')

GRPC_INCLUDE = os.path.join(GRPC_ROOT, 'include')
GRPC_PYTHON_INCLUDE = os.path.join(GRPC_PYTHON_ROOT, 'grpc_root', 'include')

BAZEL_DEPS = os.path.join(GRPC_ROOT, 'tools', 'distrib', 'python',
                          'bazel_deps.sh')
BAZEL_DEPS_PROTOC_LIB_QUERY = '//:protoc_lib'
BAZEL_DEPS_COMMON_PROTOS_QUERY = '//:well_known_protos'


def protobuf_submodule_commit_hash():
    """Gets the commit hash for the HEAD of the protobuf submodule currently
     checked out."""
    cwd = os.getcwd()
    os.chdir(GRPC_PROTOBUF_SUBMODULE_ROOT)
    output = subprocess.check_output(['git', 'rev-parse', 'HEAD'])
    os.chdir(cwd)
    return output.decode("ascii").splitlines()[0].strip()


def bazel_query(query):
    print('Running "bazel query %s"' % query)
    output = subprocess.check_output([BAZEL_DEPS, query])
    return output.decode("ascii").splitlines()


def get_deps():
    """Write the result of the bazel query `query` against protobuf to
     `out_file`."""
    cc_files_output = bazel_query(BAZEL_DEPS_PROTOC_LIB_QUERY)
    cc_files = [
        name[len(PROTOBUF_CC_PREFIX):]
        for name in cc_files_output
        if name.endswith('.cc') and name.startswith(PROTOBUF_CC_PREFIX)
    ]
    proto_files_output = bazel_query(BAZEL_DEPS_COMMON_PROTOS_QUERY)
    proto_files = [
        name[len(PROTOBUF_PROTO_PREFIX):]
        for name in proto_files_output
        if name.endswith('.proto') and name.startswith(PROTOBUF_PROTO_PREFIX)
    ]
    commit_hash = protobuf_submodule_commit_hash()
    deps_file_content = DEPS_FILE_CONTENT.format(
        cc_files=cc_files,
        proto_files=proto_files,
        cc_include=repr(GRPC_PYTHON_PROTOBUF_RELATIVE_ROOT),
        proto_include=repr(GRPC_PYTHON_PROTOBUF_RELATIVE_ROOT),
        commit_hash=COMMIT_HASH_PREFIX + commit_hash + COMMIT_HASH_SUFFIX)
    return deps_file_content


def long_path(path):
    if os.name == 'nt':
        return '\\\\?\\' + path
    else:
        return path


def main():
    os.chdir(GRPC_ROOT)

    for source, target in [(GRPC_PROTOBUF, GRPC_PYTHON_PROTOBUF),
                           (GRPC_PROTOC_PLUGINS, GRPC_PYTHON_PROTOC_PLUGINS),
                           (GRPC_INCLUDE, GRPC_PYTHON_INCLUDE)]:
        for source_dir, _, files in os.walk(source):
            target_dir = os.path.abspath(
                os.path.join(target, os.path.relpath(source_dir, source)))
            try:
                os.makedirs(target_dir)
            except OSError as error:
                if error.errno != errno.EEXIST:
                    raise
            for relative_file in files:
                source_file = os.path.abspath(
                    os.path.join(source_dir, relative_file))
                target_file = os.path.abspath(
                    os.path.join(target_dir, relative_file))
                shutil.copyfile(source_file, target_file)

    try:
        print('Invoking "bazel query" to gather the protobuf dependencies.')
        protoc_lib_deps_content = get_deps()
    except Exception as error:
        # We allow this script to succeed even if we couldn't get the dependencies,
        # as then we can assume that even without a successful bazel run the
        # dependencies currently in source control are 'good enough'.
        sys.stderr.write("Got non-fatal error:\n")
        traceback.print_exc(file=sys.stderr)
        return
    # If we successfully got the dependencies, truncate and rewrite the deps file.
    with open(GRPC_PYTHON_PROTOC_LIB_DEPS, 'w') as deps_file:
        deps_file.write(protoc_lib_deps_content)
    print('File "%s" updated.' % GRPC_PYTHON_PROTOC_LIB_DEPS)
    print('Done.')


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
#
# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import csv
import glob
import math
import multiprocessing
import os
import pathlib
import re
import shutil
import subprocess
import sys

sys.path.append(
    os.path.join(os.path.dirname(sys.argv[0]), '..', '..', 'run_tests',
                 'python_utils'))
import check_on_pr

argp = argparse.ArgumentParser(description='Perform diff on memory benchmarks')

argp.add_argument('-d',
                  '--diff_base',
                  type=str,
                  help='Commit or branch to compare the current one to')

argp.add_argument('-j', '--jobs', type=int, default=multiprocessing.cpu_count())

args = argp.parse_args()

_INTERESTING = {
    'client call':
        (rb'client call memory usage: ([0-9\.]+) bytes per call', float),
    'server call':
        (rb'server call memory usage: ([0-9\.]+) bytes per call', float),
}

_SCENARIOS = {
    'default': [],
    'minstack': ['--minstack'],
}


def _run():
    """Build with Bazel, then run, and extract interesting lines from the output."""
    subprocess.check_call([
        'tools/bazel', 'build', '-c', 'opt',
        'test/core/memory_usage/memory_usage_test'
    ])
    ret = {}
    for scenario, extra_args in _SCENARIOS.items():
        try:
            output = subprocess.check_output([
                'bazel-bin/test/core/memory_usage/memory_usage_test',
                '--warmup=10000',
                '--benchmark=50000',
            ] + extra_args)
        except subprocess.CalledProcessError as e:
            print('Error running benchmark:', e)
            continue
        for line in output.splitlines():
            for key, (pattern, conversion) in _INTERESTING.items():
                m = re.match(pattern, line)
                if m:
                    ret[scenario + ': ' + key] = conversion(m.group(1))
    return ret


cur = _run()
old = None

if args.diff_base:
    where_am_i = subprocess.check_output(
        ['git', 'rev-parse', '--abbrev-ref', 'HEAD']).decode().strip()
    # checkout the diff base (="old")
    subprocess.check_call(['git', 'checkout', args.diff_base])
    try:
        old = _run()
    finally:
        # restore the original revision (="cur")
        subprocess.check_call(['git', 'checkout', where_am_i])

text = ''
if old is None:
    print(cur)
    for key, value in sorted(cur.items()):
        text += '{}: {}\n'.format(key, value)
else:
    print(cur, old)
    diff_size = 0
    for scenario in _SCENARIOS.keys():
        for key, value in sorted(_INTERESTING.items()):
            key = scenario + ': ' + key
            if key in cur:
                if key not in old:
                    text += '{}: {}\n'.format(key, cur[key])
                else:
                    diff_size += cur[key] - old[key]
                    text += '{}: {} -> {}\n'.format(key, old[key], cur[key])

    print("DIFF_SIZE: %f" % diff_size)
    check_on_pr.label_increase_decrease_on_pr('per-call-memory', diff_size, 64)

print(text)
check_on_pr.check_on_pr('Memory Difference', '```\n%s\n```' % text)



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Python utility to build opt and counters benchmarks """

import argparse
import multiprocessing
import os
import shutil
import subprocess

import bm_constants


def _args():
    argp = argparse.ArgumentParser(description='Builds microbenchmarks')
    argp.add_argument('-b',
                      '--benchmarks',
                      nargs='+',
                      choices=bm_constants._AVAILABLE_BENCHMARK_TESTS,
                      default=bm_constants._AVAILABLE_BENCHMARK_TESTS,
                      help='Which benchmarks to build')
    argp.add_argument(
        '-j',
        '--jobs',
        type=int,
        default=multiprocessing.cpu_count(),
        help=
        'Deprecated. Bazel chooses number of CPUs to build with automatically.')
    argp.add_argument(
        '-n',
        '--name',
        type=str,
        help=
        'Unique name of this build. To be used as a handle to pass to the other bm* scripts'
    )
    argp.add_argument('--counters', dest='counters', action='store_true')
    argp.add_argument('--no-counters', dest='counters', action='store_false')
    argp.set_defaults(counters=True)
    args = argp.parse_args()
    assert args.name
    return args


def _build_cmd(cfg, benchmarks):
    bazel_targets = [
        '//test/cpp/microbenchmarks:%s' % benchmark for benchmark in benchmarks
    ]
    # --dynamic_mode=off makes sure that we get a monolithic binary that can be safely
    # moved outside of the bazel-bin directory
    return ['tools/bazel', 'build',
            '--config=%s' % cfg, '--dynamic_mode=off'] + bazel_targets


def _build_config_and_copy(cfg, benchmarks, dest_dir):
    """Build given config and copy resulting binaries to dest_dir/CONFIG"""
    subprocess.check_call(_build_cmd(cfg, benchmarks))
    cfg_dir = dest_dir + '/%s' % cfg
    os.makedirs(cfg_dir)
    subprocess.check_call(['cp'] + [
        'bazel-bin/test/cpp/microbenchmarks/%s' % benchmark
        for benchmark in benchmarks
    ] + [cfg_dir])


def build(name, benchmarks, jobs, counters):
    dest_dir = 'bm_diff_%s' % name
    shutil.rmtree(dest_dir, ignore_errors=True)
    _build_config_and_copy('opt', benchmarks, dest_dir)
    if counters:
        _build_config_and_copy('counters', benchmarks, dest_dir)


if __name__ == '__main__':
    args = _args()
    build(args.name, args.benchmarks, args.jobs, args.counters)



#!/usr/bin/env python3
# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Definition of targets to build distribution packages."""

import os.path
import sys

sys.path.insert(0, os.path.abspath('..'))
import python_utils.jobset as jobset


def create_docker_jobspec(name,
                          dockerfile_dir,
                          shell_command,
                          environ={},
                          flake_retries=0,
                          timeout_retries=0):
    """Creates jobspec for a task running under docker."""
    environ = environ.copy()

    docker_args = []
    for k, v in list(environ.items()):
        docker_args += ['-e', '%s=%s' % (k, v)]
    docker_env = {
        'DOCKERFILE_DIR': dockerfile_dir,
        'DOCKER_RUN_SCRIPT': 'tools/run_tests/dockerize/docker_run.sh',
        'DOCKER_RUN_SCRIPT_COMMAND': shell_command,
        'OUTPUT_DIR': 'artifacts'
    }
    jobspec = jobset.JobSpec(
        cmdline=['tools/run_tests/dockerize/build_and_run_docker.sh'] +
        docker_args,
        environ=docker_env,
        shortname='build_package.%s' % (name),
        timeout_seconds=30 * 60,
        flake_retries=flake_retries,
        timeout_retries=timeout_retries)
    return jobspec


def create_jobspec(name,
                   cmdline,
                   environ=None,
                   cwd=None,
                   shell=False,
                   flake_retries=0,
                   timeout_retries=0,
                   cpu_cost=1.0):
    """Creates jobspec."""
    jobspec = jobset.JobSpec(cmdline=cmdline,
                             environ=environ,
                             cwd=cwd,
                             shortname='build_package.%s' % (name),
                             timeout_seconds=10 * 60,
                             flake_retries=flake_retries,
                             timeout_retries=timeout_retries,
                             cpu_cost=cpu_cost,
                             shell=shell)
    return jobspec


class CSharpPackage:
    """Builds C# packages."""

    def __init__(self, platform):
        self.platform = platform
        self.labels = ['package', 'csharp', self.platform]
        self.name = 'csharp_package_nuget_%s' % self.platform
        self.labels += ['nuget']

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as there is little opportunity for parallelizing
        environ = {
            'GRPC_CSHARP_BUILD_SINGLE_PLATFORM_NUGET':
                os.getenv('GRPC_CSHARP_BUILD_SINGLE_PLATFORM_NUGET', '')
        }

        build_script = 'src/csharp/build_nuget.sh'

        if self.platform == 'linux':
            return create_docker_jobspec(
                self.name,
                'tools/dockerfile/test/csharp_debian11_x64',
                build_script,
                environ=environ)
        else:
            repo_root = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                     '..', '..', '..')
            environ['EXTERNAL_GIT_ROOT'] = repo_root
            return create_jobspec(self.name, ['bash', build_script],
                                  environ=environ)

    def __str__(self):
        return self.name


class RubyPackage:
    """Collects ruby gems created in the artifact phase"""

    def __init__(self):
        self.name = 'ruby_package'
        self.labels = ['package', 'ruby', 'linux']

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as this step simply collects preexisting artifacts
        return create_docker_jobspec(
            self.name, 'tools/dockerfile/grpc_artifact_centos6_x64',
            'tools/run_tests/artifacts/build_package_ruby.sh')


class PythonPackage:
    """Collects python eggs and wheels created in the artifact phase"""

    def __init__(self):
        self.name = 'python_package'
        self.labels = ['package', 'python', 'linux']

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as this step simply collects preexisting artifacts
        # since the python package build does very little, we can use virtually
        # any image that has new-enough python, so reusing one of the images used
        # for artifact building seems natural.
        return create_docker_jobspec(
            self.name,
            'tools/dockerfile/grpc_artifact_python_manylinux2014_x64',
            'tools/run_tests/artifacts/build_package_python.sh',
            environ={'PYTHON': '/opt/python/cp39-cp39/bin/python'})


class PHPPackage:
    """Copy PHP PECL package artifact"""

    def __init__(self):
        self.name = 'php_package'
        self.labels = ['package', 'php', 'linux']

    def pre_build_jobspecs(self):
        return []

    def build_jobspec(self, inner_jobs=None):
        del inner_jobs  # arg unused as this step simply collects preexisting artifacts
        return create_docker_jobspec(
            self.name, 'tools/dockerfile/grpc_artifact_centos6_x64',
            'tools/run_tests/artifacts/build_package_php.sh')


def targets():
    """Gets list of supported targets"""
    return [
        CSharpPackage('linux'),
        CSharpPackage('macos'),
        CSharpPackage('windows'),
        RubyPackage(),
        PythonPackage(),
        PHPPackage()
    ]



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Autogenerated by tools/codegen/core/gen_stats_data.py

import massage_qps_stats_helpers


def massage_qps_stats(scenario_result):
    for stats in scenario_result["serverStats"] + scenario_result["clientStats"]:
        if "coreStats" in stats:
            # Get rid of the "coreStats" element and replace it by statistics
            # that correspond to columns in the bigquery schema.
            core_stats = stats["coreStats"]
            del stats["coreStats"]
            stats[
                "core_client_calls_created"] = massage_qps_stats_helpers.counter(
                    core_stats, "client_calls_created")
            stats[
                "core_server_calls_created"] = massage_qps_stats_helpers.counter(
                    core_stats, "server_calls_created")
            stats["core_cqs_created"] = massage_qps_stats_helpers.counter(
                core_stats, "cqs_created")
            stats[
                "core_client_channels_created"] = massage_qps_stats_helpers.counter(
                    core_stats, "client_channels_created")
            stats[
                "core_client_subchannels_created"] = massage_qps_stats_helpers.counter(
                    core_stats, "client_subchannels_created")
            stats[
                "core_server_channels_created"] = massage_qps_stats_helpers.counter(
                    core_stats, "server_channels_created")
            stats["core_syscall_poll"] = massage_qps_stats_helpers.counter(
                core_stats, "syscall_poll")
            stats["core_syscall_wait"] = massage_qps_stats_helpers.counter(
                core_stats, "syscall_wait")
            stats["core_pollset_kick"] = massage_qps_stats_helpers.counter(
                core_stats, "pollset_kick")
            stats[
                "core_pollset_kicked_without_poller"] = massage_qps_stats_helpers.counter(
                    core_stats, "pollset_kicked_without_poller")
            stats[
                "core_pollset_kicked_again"] = massage_qps_stats_helpers.counter(
                    core_stats, "pollset_kicked_again")
            stats[
                "core_pollset_kick_wakeup_fd"] = massage_qps_stats_helpers.counter(
                    core_stats, "pollset_kick_wakeup_fd")
            stats[
                "core_pollset_kick_wakeup_cv"] = massage_qps_stats_helpers.counter(
                    core_stats, "pollset_kick_wakeup_cv")
            stats[
                "core_pollset_kick_own_thread"] = massage_qps_stats_helpers.counter(
                    core_stats, "pollset_kick_own_thread")
            stats[
                "core_histogram_slow_lookups"] = massage_qps_stats_helpers.counter(
                    core_stats, "histogram_slow_lookups")
            stats["core_syscall_write"] = massage_qps_stats_helpers.counter(
                core_stats, "syscall_write")
            stats["core_syscall_read"] = massage_qps_stats_helpers.counter(
                core_stats, "syscall_read")
            stats[
                "core_tcp_backup_pollers_created"] = massage_qps_stats_helpers.counter(
                    core_stats, "tcp_backup_pollers_created")
            stats[
                "core_tcp_backup_poller_polls"] = massage_qps_stats_helpers.counter(
                    core_stats, "tcp_backup_poller_polls")
            stats["core_http2_op_batches"] = massage_qps_stats_helpers.counter(
                core_stats, "http2_op_batches")
            stats["core_http2_op_cancel"] = massage_qps_stats_helpers.counter(
                core_stats, "http2_op_cancel")
            stats[
                "core_http2_op_send_initial_metadata"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_op_send_initial_metadata")
            stats[
                "core_http2_op_send_message"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_op_send_message")
            stats[
                "core_http2_op_send_trailing_metadata"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_op_send_trailing_metadata")
            stats[
                "core_http2_op_recv_initial_metadata"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_op_recv_initial_metadata")
            stats[
                "core_http2_op_recv_message"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_op_recv_message")
            stats[
                "core_http2_op_recv_trailing_metadata"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_op_recv_trailing_metadata")
            stats[
                "core_http2_settings_writes"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_settings_writes")
            stats["core_http2_pings_sent"] = massage_qps_stats_helpers.counter(
                core_stats, "http2_pings_sent")
            stats[
                "core_http2_writes_begun"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_writes_begun")
            stats[
                "core_http2_writes_offloaded"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_writes_offloaded")
            stats[
                "core_http2_writes_continued"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_writes_continued")
            stats[
                "core_http2_partial_writes"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_partial_writes")
            stats[
                "core_http2_initiate_write_due_to_initial_write"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_initial_write")
            stats[
                "core_http2_initiate_write_due_to_start_new_stream"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_start_new_stream")
            stats[
                "core_http2_initiate_write_due_to_send_message"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_send_message")
            stats[
                "core_http2_initiate_write_due_to_send_initial_metadata"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_send_initial_metadata")
            stats[
                "core_http2_initiate_write_due_to_send_trailing_metadata"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_send_trailing_metadata")
            stats[
                "core_http2_initiate_write_due_to_retry_send_ping"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_retry_send_ping")
            stats[
                "core_http2_initiate_write_due_to_continue_pings"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_continue_pings")
            stats[
                "core_http2_initiate_write_due_to_goaway_sent"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_goaway_sent")
            stats[
                "core_http2_initiate_write_due_to_rst_stream"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_rst_stream")
            stats[
                "core_http2_initiate_write_due_to_close_from_api"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_close_from_api")
            stats[
                "core_http2_initiate_write_due_to_stream_flow_control"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_stream_flow_control")
            stats[
                "core_http2_initiate_write_due_to_transport_flow_control"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_transport_flow_control")
            stats[
                "core_http2_initiate_write_due_to_send_settings"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_send_settings")
            stats[
                "core_http2_initiate_write_due_to_settings_ack"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_settings_ack")
            stats[
                "core_http2_initiate_write_due_to_bdp_estimator_ping"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_bdp_estimator_ping")
            stats[
                "core_http2_initiate_write_due_to_flow_control_unstalled_by_setting"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_flow_control_unstalled_by_setting"
                )
            stats[
                "core_http2_initiate_write_due_to_flow_control_unstalled_by_update"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_flow_control_unstalled_by_update"
                )
            stats[
                "core_http2_initiate_write_due_to_application_ping"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_application_ping")
            stats[
                "core_http2_initiate_write_due_to_keepalive_ping"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_keepalive_ping")
            stats[
                "core_http2_initiate_write_due_to_transport_flow_control_unstalled"] = massage_qps_stats_helpers.counter(
                    core_stats,
                    "http2_initiate_write_due_to_transport_flow_control_unstalled"
                )
            stats[
                "core_http2_initiate_write_due_to_ping_response"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_ping_response")
            stats[
                "core_http2_initiate_write_due_to_force_rst_stream"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_initiate_write_due_to_force_rst_stream")
            stats[
                "core_http2_spurious_writes_begun"] = massage_qps_stats_helpers.counter(
                    core_stats, "http2_spurious_writes_begun")
            stats[
                "core_hpack_recv_indexed"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_indexed")
            stats[
                "core_hpack_recv_lithdr_incidx"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_lithdr_incidx")
            stats[
                "core_hpack_recv_lithdr_incidx_v"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_lithdr_incidx_v")
            stats[
                "core_hpack_recv_lithdr_notidx"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_lithdr_notidx")
            stats[
                "core_hpack_recv_lithdr_notidx_v"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_lithdr_notidx_v")
            stats[
                "core_hpack_recv_lithdr_nvridx"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_lithdr_nvridx")
            stats[
                "core_hpack_recv_lithdr_nvridx_v"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_lithdr_nvridx_v")
            stats[
                "core_hpack_recv_uncompressed"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_uncompressed")
            stats[
                "core_hpack_recv_huffman"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_huffman")
            stats["core_hpack_recv_binary"] = massage_qps_stats_helpers.counter(
                core_stats, "hpack_recv_binary")
            stats[
                "core_hpack_recv_binary_base64"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_recv_binary_base64")
            stats[
                "core_hpack_send_indexed"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_indexed")
            stats[
                "core_hpack_send_lithdr_incidx"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_lithdr_incidx")
            stats[
                "core_hpack_send_lithdr_incidx_v"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_lithdr_incidx_v")
            stats[
                "core_hpack_send_lithdr_notidx"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_lithdr_notidx")
            stats[
                "core_hpack_send_lithdr_notidx_v"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_lithdr_notidx_v")
            stats[
                "core_hpack_send_lithdr_nvridx"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_lithdr_nvridx")
            stats[
                "core_hpack_send_lithdr_nvridx_v"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_lithdr_nvridx_v")
            stats[
                "core_hpack_send_uncompressed"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_uncompressed")
            stats[
                "core_hpack_send_huffman"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_huffman")
            stats["core_hpack_send_binary"] = massage_qps_stats_helpers.counter(
                core_stats, "hpack_send_binary")
            stats[
                "core_hpack_send_binary_base64"] = massage_qps_stats_helpers.counter(
                    core_stats, "hpack_send_binary_base64")
            stats[
                "core_combiner_locks_initiated"] = massage_qps_stats_helpers.counter(
                    core_stats, "combiner_locks_initiated")
            stats[
                "core_combiner_locks_scheduled_items"] = massage_qps_stats_helpers.counter(
                    core_stats, "combiner_locks_scheduled_items")
            stats[
                "core_combiner_locks_scheduled_final_items"] = massage_qps_stats_helpers.counter(
                    core_stats, "combiner_locks_scheduled_final_items")
            stats[
                "core_combiner_locks_offloaded"] = massage_qps_stats_helpers.counter(
                    core_stats, "combiner_locks_offloaded")
            stats[
                "core_call_combiner_locks_initiated"] = massage_qps_stats_helpers.counter(
                    core_stats, "call_combiner_locks_initiated")
            stats[
                "core_call_combiner_locks_scheduled_items"] = massage_qps_stats_helpers.counter(
                    core_stats, "call_combiner_locks_scheduled_items")
            stats[
                "core_call_combiner_set_notify_on_cancel"] = massage_qps_stats_helpers.counter(
                    core_stats, "call_combiner_set_notify_on_cancel")
            stats[
                "core_call_combiner_cancelled"] = massage_qps_stats_helpers.counter(
                    core_stats, "call_combiner_cancelled")
            stats[
                "core_executor_scheduled_short_items"] = massage_qps_stats_helpers.counter(
                    core_stats, "executor_scheduled_short_items")
            stats[
                "core_executor_scheduled_long_items"] = massage_qps_stats_helpers.counter(
                    core_stats, "executor_scheduled_long_items")
            stats[
                "core_executor_scheduled_to_self"] = massage_qps_stats_helpers.counter(
                    core_stats, "executor_scheduled_to_self")
            stats[
                "core_executor_wakeup_initiated"] = massage_qps_stats_helpers.counter(
                    core_stats, "executor_wakeup_initiated")
            stats[
                "core_executor_queue_drained"] = massage_qps_stats_helpers.counter(
                    core_stats, "executor_queue_drained")
            stats[
                "core_executor_push_retries"] = massage_qps_stats_helpers.counter(
                    core_stats, "executor_push_retries")
            stats[
                "core_server_requested_calls"] = massage_qps_stats_helpers.counter(
                    core_stats, "server_requested_calls")
            stats[
                "core_server_slowpath_requests_queued"] = massage_qps_stats_helpers.counter(
                    core_stats, "server_slowpath_requests_queued")
            stats[
                "core_cq_ev_queue_trylock_failures"] = massage_qps_stats_helpers.counter(
                    core_stats, "cq_ev_queue_trylock_failures")
            stats[
                "core_cq_ev_queue_trylock_successes"] = massage_qps_stats_helpers.counter(
                    core_stats, "cq_ev_queue_trylock_successes")
            stats[
                "core_cq_ev_queue_transient_pop_failures"] = massage_qps_stats_helpers.counter(
                    core_stats, "cq_ev_queue_transient_pop_failures")
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "call_initial_size")
            stats["core_call_initial_size"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_call_initial_size_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_call_initial_size_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_call_initial_size_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_call_initial_size_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "poll_events_returned")
            stats["core_poll_events_returned"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_poll_events_returned_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_poll_events_returned_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_poll_events_returned_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_poll_events_returned_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "tcp_write_size")
            stats["core_tcp_write_size"] = ",".join("%f" % x for x in h.buckets)
            stats["core_tcp_write_size_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_tcp_write_size_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_tcp_write_size_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_tcp_write_size_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "tcp_write_iov_size")
            stats["core_tcp_write_iov_size"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_tcp_write_iov_size_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_tcp_write_iov_size_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_tcp_write_iov_size_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_tcp_write_iov_size_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats, "tcp_read_size")
            stats["core_tcp_read_size"] = ",".join("%f" % x for x in h.buckets)
            stats["core_tcp_read_size_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_tcp_read_size_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_tcp_read_size_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_tcp_read_size_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "tcp_read_offer")
            stats["core_tcp_read_offer"] = ",".join("%f" % x for x in h.buckets)
            stats["core_tcp_read_offer_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_tcp_read_offer_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_tcp_read_offer_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_tcp_read_offer_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "tcp_read_offer_iov_size")
            stats["core_tcp_read_offer_iov_size"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_tcp_read_offer_iov_size_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_tcp_read_offer_iov_size_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_tcp_read_offer_iov_size_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_tcp_read_offer_iov_size_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "http2_send_message_size")
            stats["core_http2_send_message_size"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_http2_send_message_size_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_http2_send_message_size_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_http2_send_message_size_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_http2_send_message_size_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(
                core_stats, "http2_send_initial_metadata_per_write")
            stats["core_http2_send_initial_metadata_per_write"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_http2_send_initial_metadata_per_write_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_http2_send_initial_metadata_per_write_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_http2_send_initial_metadata_per_write_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_http2_send_initial_metadata_per_write_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(
                core_stats, "http2_send_message_per_write")
            stats["core_http2_send_message_per_write"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_http2_send_message_per_write_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_http2_send_message_per_write_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_http2_send_message_per_write_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_http2_send_message_per_write_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(
                core_stats, "http2_send_trailing_metadata_per_write")
            stats["core_http2_send_trailing_metadata_per_write"] = ",".join(
                "%f" % x for x in h.buckets)
            stats[
                "core_http2_send_trailing_metadata_per_write_bkts"] = ",".join(
                    "%f" % x for x in h.boundaries)
            stats[
                "core_http2_send_trailing_metadata_per_write_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_http2_send_trailing_metadata_per_write_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_http2_send_trailing_metadata_per_write_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(
                core_stats, "http2_send_flowctl_per_write")
            stats["core_http2_send_flowctl_per_write"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_http2_send_flowctl_per_write_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_http2_send_flowctl_per_write_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_http2_send_flowctl_per_write_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_http2_send_flowctl_per_write_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)
            h = massage_qps_stats_helpers.histogram(core_stats,
                                                    "server_cqs_checked")
            stats["core_server_cqs_checked"] = ",".join(
                "%f" % x for x in h.buckets)
            stats["core_server_cqs_checked_bkts"] = ",".join(
                "%f" % x for x in h.boundaries)
            stats[
                "core_server_cqs_checked_50p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 50, h.boundaries)
            stats[
                "core_server_cqs_checked_95p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 95, h.boundaries)
            stats[
                "core_server_cqs_checked_99p"] = massage_qps_stats_helpers.percentile(
                    h.buckets, 99, h.boundaries)



#!/usr/bin/env python2.7

# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# TODO: This should ideally be in upb submodule to avoid hardcoding this here.

import re
import os
import sys
import yaml

out = {}

try:
    out['libs'] = [{
        'name': 'upb',
        'build': 'all',
        'language': 'c',
        'src': [
            "third_party/upb/third_party/utf8_range/naive.c",
            "third_party/upb/third_party/utf8_range/range2-neon.c",
            "third_party/upb/third_party/utf8_range/range2-sse.c",
            "third_party/upb/upb/decode_fast.c",
            "third_party/upb/upb/decode.c",
            "third_party/upb/upb/def.c",
            "third_party/upb/upb/encode.c",
            "third_party/upb/upb/json_encode.c",
            "third_party/upb/upb/msg.c",
            "third_party/upb/upb/reflection.c",
            "third_party/upb/upb/table.c",
            "third_party/upb/upb/text_encode.c",
            "third_party/upb/upb/upb.c",
            "src/core/ext/upb-generated/google/protobuf/descriptor.upb.c",
            "src/core/ext/upbdefs-generated/google/protobuf/descriptor.upbdefs.c",
        ],
        'headers': [
            "third_party/upb/third_party/utf8_range/utf8_range.h",
            "third_party/upb/upb/decode_fast.h",
            "third_party/upb/upb/decode_internal.h",
            "third_party/upb/upb/decode.h",
            "third_party/upb/upb/def.h",
            "third_party/upb/upb/def.hpp",
            "third_party/upb/upb/encode.h",
            "third_party/upb/upb/json_encode.h",
            "third_party/upb/upb/msg_internal.h",
            "third_party/upb/upb/msg.h",
            "third_party/upb/upb/port_def.inc",
            "third_party/upb/upb/port_undef.inc",
            "third_party/upb/upb/reflection.h",
            "third_party/upb/upb/reflection.hpp",
            "third_party/upb/upb/table_internal.h",
            "third_party/upb/upb/text_encode.h",
            "third_party/upb/upb/upb_internal.h",
            "third_party/upb/upb/upb.h",
            "third_party/upb/upb/upb.hpp",
            "src/core/ext/upb-generated/google/protobuf/descriptor.upb.h",
            "src/core/ext/upbdefs-generated/google/protobuf/descriptor.upbdefs.h",
        ],
        'secure': False,
    }]
except:
    pass

print(yaml.dump(out))



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC interoperability test server."""

import argparse
from concurrent import futures
import logging

import grpc

from src.proto.grpc.testing import test_pb2_grpc
from tests.interop import resources
from tests.interop import service
from tests.unit import test_common

logging.basicConfig()
_LOGGER = logging.getLogger(__name__)


def parse_interop_server_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument('--port',
                        type=int,
                        required=True,
                        help='the port on which to serve')
    parser.add_argument('--use_tls',
                        default=False,
                        type=resources.parse_bool,
                        help='require a secure connection')
    parser.add_argument('--use_alts',
                        default=False,
                        type=resources.parse_bool,
                        help='require an ALTS connection')
    return parser.parse_args()


def get_server_credentials(use_tls):
    if use_tls:
        private_key = resources.private_key()
        certificate_chain = resources.certificate_chain()
        return grpc.ssl_server_credentials(((private_key, certificate_chain),))
    else:
        return grpc.alts_server_credentials()


def serve():
    args = parse_interop_server_arguments()

    server = test_common.test_server()
    test_pb2_grpc.add_TestServiceServicer_to_server(service.TestService(),
                                                    server)
    if args.use_tls or args.use_alts:
        credentials = get_server_credentials(args.use_tls)
        server.add_secure_port('[::]:{}'.format(args.port), credentials)
    else:
        server.add_insecure_port('[::]:{}'.format(args.port))

    server.start()
    _LOGGER.info('Server serving.')
    server.wait_for_termination()
    _LOGGER.info('Server stopped; exiting.')


if __name__ == '__main__':
    serve()



#!/usr/bin/env python3

# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), '../../..'))

BAD_REGEXES = [
    (r'\n#include "include/(.*)"', r'\n#include <\1>'),
    (r'\n#include "grpc(.*)"', r'\n#include <grpc\1>'),
]

fix = sys.argv[1:] == ['--fix']
if fix:
    print("FIXING!")


def check_include_style(directory_root):
    bad_files = []
    for root, dirs, files in os.walk(directory_root):
        for filename in files:
            path = os.path.join(root, filename)
            if os.path.splitext(path)[1] not in ['.c', '.cc', '.h']:
                continue
            if filename.endswith('.pb.h') or filename.endswith('.pb.c'):
                continue
            # Skip check for upb generated code.
            if (filename.endswith('.upb.h') or filename.endswith('.upb.c') or
                    filename.endswith('.upbdefs.h') or
                    filename.endswith('.upbdefs.c')):
                continue
            with open(path) as f:
                text = f.read()
            original = text
            for regex, replace in BAD_REGEXES:
                text = re.sub(regex, replace, text)
            if text != original:
                bad_files.append(path)
                if fix:
                    with open(path, 'w') as f:
                        f.write(text)
    return bad_files


all_bad_files = []
all_bad_files += check_include_style(os.path.join('src', 'core'))
all_bad_files += check_include_style(os.path.join('src', 'cpp'))
all_bad_files += check_include_style(os.path.join('test', 'core'))
all_bad_files += check_include_style(os.path.join('test', 'cpp'))
all_bad_files += check_include_style(os.path.join('include', 'grpc'))
all_bad_files += check_include_style(os.path.join('include', 'grpcpp'))

if all_bad_files:
    for f in all_bad_files:
        print("%s has badly formed grpc system header files" % f)
    sys.exit(1)



# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

from absl import app
from absl import flags

from framework import xds_flags
from framework import xds_k8s_flags
from framework.infrastructure import gcp
from framework.infrastructure import k8s
from framework.test_app import server_app

logger = logging.getLogger(__name__)
# Flags
_CMD = flags.DEFINE_enum('cmd',
                         default='run',
                         enum_values=['run', 'cleanup'],
                         help='Command')
_SECURE = flags.DEFINE_bool("secure",
                            default=False,
                            help="Run server in the secure mode")
_REUSE_NAMESPACE = flags.DEFINE_bool("reuse_namespace",
                                     default=True,
                                     help="Use existing namespace if exists")
_CLEANUP_NAMESPACE = flags.DEFINE_bool(
    "cleanup_namespace",
    default=False,
    help="Delete namespace during resource cleanup")
flags.adopt_module_key_flags(xds_flags)
flags.adopt_module_key_flags(xds_k8s_flags)
# Running outside of a test suite, so require explicit resource_suffix.
flags.mark_flag_as_required("resource_suffix")

KubernetesServerRunner = server_app.KubernetesServerRunner


def main(argv):
    if len(argv) > 1:
        raise app.UsageError('Too many command-line arguments.')

    project: str = xds_flags.PROJECT.value
    # GCP Service Account email
    gcp_service_account: str = xds_k8s_flags.GCP_SERVICE_ACCOUNT.value

    # Resource names.
    resource_prefix: str = xds_flags.RESOURCE_PREFIX.value
    resource_suffix: str = xds_flags.RESOURCE_SUFFIX.value

    # KubernetesServerRunner arguments.
    runner_kwargs = dict(
        deployment_name=xds_flags.SERVER_NAME.value,
        image_name=xds_k8s_flags.SERVER_IMAGE.value,
        td_bootstrap_image=xds_k8s_flags.TD_BOOTSTRAP_IMAGE.value,
        gcp_project=project,
        gcp_api_manager=gcp.api.GcpApiManager(),
        gcp_service_account=gcp_service_account,
        network=xds_flags.NETWORK.value,
        reuse_namespace=_REUSE_NAMESPACE.value)

    if _SECURE.value:
        runner_kwargs.update(
            xds_server_uri=xds_flags.XDS_SERVER_URI.value,
            deployment_template='server-secure.deployment.yaml')

    k8s_api_manager = k8s.KubernetesApiManager(xds_k8s_flags.KUBE_CONTEXT.value)
    server_namespace = KubernetesServerRunner.make_namespace_name(
        resource_prefix, resource_suffix)
    server_runner = KubernetesServerRunner(
        k8s.KubernetesNamespace(k8s_api_manager, server_namespace),
        **runner_kwargs)

    if _CMD.value == 'run':
        logger.info('Run server, secure_mode=%s', _SECURE.value)
        server_runner.run(
            test_port=xds_flags.SERVER_PORT.value,
            maintenance_port=xds_flags.SERVER_MAINTENANCE_PORT.value,
            secure_mode=_SECURE.value)

    elif _CMD.value == 'cleanup':
        logger.info('Cleanup server')
        server_runner.cleanup(force=True,
                              force_namespace=_CLEANUP_NAMESPACE.value)


if __name__ == '__main__':
    app.run(main)



# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import contextlib
import datetime
import logging
import pathlib
from typing import Dict, Optional
import urllib.parse

import mako.template
import yaml

import framework.helpers.datetime
import framework.helpers.highlighter
from framework.infrastructure import gcp
from framework.infrastructure import k8s

logger = logging.getLogger(__name__)

# Type aliases
_HighlighterYaml = framework.helpers.highlighter.HighlighterYaml
_helper_datetime = framework.helpers.datetime
timedelta = datetime.timedelta


def _logs_explorer_query(query: Dict[str, str]) -> str:
    return '\n'.join(f'{k}="{v}"' for k, v in query.items())


def _logs_explorer_request(req: Dict[str, str]) -> str:
    return ';'.join(f'{k}={_logs_explorer_quote(v)}' for k, v in req.items())


def _logs_explorer_quote(value: str):
    return urllib.parse.quote_plus(value, safe=':')


class RunnerError(Exception):
    """Error running app"""


class KubernetesBaseRunner:
    TEMPLATE_DIR_NAME = 'kubernetes-manifests'
    TEMPLATE_DIR_RELATIVE_PATH = f'../../{TEMPLATE_DIR_NAME}'
    ROLE_WORKLOAD_IDENTITY_USER = 'roles/iam.workloadIdentityUser'

    def __init__(self,
                 k8s_namespace,
                 namespace_template=None,
                 reuse_namespace=False):
        self._highlighter = _HighlighterYaml()

        # Kubernetes namespaced resources manager
        self.k8s_namespace: k8s.KubernetesNamespace = k8s_namespace
        self.reuse_namespace = reuse_namespace
        self.namespace_template = namespace_template or 'namespace.yaml'

        # Mutable state
        self.namespace: Optional[k8s.V1Namespace] = None

    def run(self, **kwargs):
        del kwargs
        if self.reuse_namespace:
            self.namespace = self._reuse_namespace()
        if not self.namespace:
            self.namespace = self._create_namespace(
                self.namespace_template, namespace_name=self.k8s_namespace.name)

    def cleanup(self, *, force=False):
        if (self.namespace and not self.reuse_namespace) or force:
            self.delete_namespace()
            self.namespace = None

    @staticmethod
    def _render_template(template_file, **kwargs):
        template = mako.template.Template(filename=str(template_file))
        return template.render(**kwargs)

    @staticmethod
    def _manifests_from_yaml_file(yaml_file):
        with open(yaml_file) as f:
            with contextlib.closing(yaml.safe_load_all(f)) as yml:
                for manifest in yml:
                    yield manifest

    @staticmethod
    def _manifests_from_str(document):
        with contextlib.closing(yaml.safe_load_all(document)) as yml:
            for manifest in yml:
                yield manifest

    @classmethod
    def _template_file_from_name(cls, template_name):
        templates_path = (pathlib.Path(__file__).parent /
                          cls.TEMPLATE_DIR_RELATIVE_PATH)
        return templates_path.joinpath(template_name).resolve()

    def _create_from_template(self, template_name, **kwargs):
        template_file = self._template_file_from_name(template_name)
        logger.debug("Loading k8s manifest template: %s", template_file)

        yaml_doc = self._render_template(template_file, **kwargs)
        logger.info("Rendered template %s/%s:\n%s", self.TEMPLATE_DIR_NAME,
                    template_name, self._highlighter.highlight(yaml_doc))

        manifests = self._manifests_from_str(yaml_doc)
        manifest = next(manifests)
        # Error out on multi-document yaml
        if next(manifests, False):
            raise RunnerError('Exactly one document expected in manifest '
                              f'{template_file}')
        k8s_objects = self.k8s_namespace.apply_manifest(manifest)
        if len(k8s_objects) != 1:
            raise RunnerError('Expected exactly one object must created from '
                              f'manifest {template_file}')

        logger.info('%s %s created', k8s_objects[0].kind,
                    k8s_objects[0].metadata.name)
        return k8s_objects[0]

    def _reuse_deployment(self, deployment_name) -> k8s.V1Deployment:
        deployment = self.k8s_namespace.get_deployment(deployment_name)
        # TODO(sergiitk): check if good or must be recreated
        return deployment

    def _reuse_service(self, service_name) -> k8s.V1Service:
        service = self.k8s_namespace.get_service(service_name)
        # TODO(sergiitk): check if good or must be recreated
        return service

    def _reuse_namespace(self) -> k8s.V1Namespace:
        return self.k8s_namespace.get()

    def _create_namespace(self, template, **kwargs) -> k8s.V1Namespace:
        namespace = self._create_from_template(template, **kwargs)
        if not isinstance(namespace, k8s.V1Namespace):
            raise RunnerError('Expected V1Namespace to be created '
                              f'from manifest {template}')
        if namespace.metadata.name != kwargs['namespace_name']:
            raise RunnerError('V1Namespace created with unexpected name: '
                              f'{namespace.metadata.name}')
        logger.debug('V1Namespace %s created at %s',
                     namespace.metadata.self_link,
                     namespace.metadata.creation_timestamp)
        return namespace

    @staticmethod
    def _get_workload_identity_member_name(project, namespace_name,
                                           service_account_name):
        """
        Returns workload identity member name used to authenticate Kubernetes
        service accounts.

        https://cloud.google.com/kubernetes-engine/docs/how-to/workload-identity
        """
        return (f'serviceAccount:{project}.svc.id.goog'
                f'[{namespace_name}/{service_account_name}]')

    def _grant_workload_identity_user(self, *, gcp_iam, gcp_service_account,
                                      service_account_name):
        workload_identity_member = self._get_workload_identity_member_name(
            gcp_iam.project, self.k8s_namespace.name, service_account_name)
        logger.info('Granting %s to %s for GCP Service Account %s',
                    self.ROLE_WORKLOAD_IDENTITY_USER, workload_identity_member,
                    gcp_service_account)

        gcp_iam.add_service_account_iam_policy_binding(
            gcp_service_account, self.ROLE_WORKLOAD_IDENTITY_USER,
            workload_identity_member)

    def _revoke_workload_identity_user(self, *, gcp_iam, gcp_service_account,
                                       service_account_name):
        workload_identity_member = self._get_workload_identity_member_name(
            gcp_iam.project, self.k8s_namespace.name, service_account_name)
        logger.info('Revoking %s from %s for GCP Service Account %s',
                    self.ROLE_WORKLOAD_IDENTITY_USER, workload_identity_member,
                    gcp_service_account)
        try:
            gcp_iam.remove_service_account_iam_policy_binding(
                gcp_service_account, self.ROLE_WORKLOAD_IDENTITY_USER,
                workload_identity_member)
        except gcp.api.Error as error:
            logger.warning('Failed  %s from %s for Service Account %s: %r',
                           self.ROLE_WORKLOAD_IDENTITY_USER,
                           workload_identity_member, gcp_service_account, error)

    def _create_service_account(self, template,
                                **kwargs) -> k8s.V1ServiceAccount:
        resource = self._create_from_template(template, **kwargs)
        if not isinstance(resource, k8s.V1ServiceAccount):
            raise RunnerError('Expected V1ServiceAccount to be created '
                              f'from manifest {template}')
        if resource.metadata.name != kwargs['service_account_name']:
            raise RunnerError('V1ServiceAccount created with unexpected name: '
                              f'{resource.metadata.name}')
        logger.debug('V1ServiceAccount %s created at %s',
                     resource.metadata.self_link,
                     resource.metadata.creation_timestamp)
        return resource

    def _create_deployment(self, template, **kwargs) -> k8s.V1Deployment:
        deployment = self._create_from_template(template, **kwargs)
        if not isinstance(deployment, k8s.V1Deployment):
            raise RunnerError('Expected V1Deployment to be created '
                              f'from manifest {template}')
        if deployment.metadata.name != kwargs['deployment_name']:
            raise RunnerError('V1Deployment created with unexpected name: '
                              f'{deployment.metadata.name}')
        logger.debug('V1Deployment %s created at %s',
                     deployment.metadata.self_link,
                     deployment.metadata.creation_timestamp)
        return deployment

    def _create_service(self, template, **kwargs) -> k8s.V1Service:
        service = self._create_from_template(template, **kwargs)
        if not isinstance(service, k8s.V1Service):
            raise RunnerError('Expected V1Service to be created '
                              f'from manifest {template}')
        if service.metadata.name != kwargs['service_name']:
            raise RunnerError('V1Service created with unexpected name: '
                              f'{service.metadata.name}')
        logger.debug('V1Service %s created at %s', service.metadata.self_link,
                     service.metadata.creation_timestamp)
        return service

    def _delete_deployment(self, name, wait_for_deletion=True):
        logger.info('Deleting deployment %s', name)
        try:
            self.k8s_namespace.delete_deployment(name)
        except k8s.ApiException as e:
            logger.info('Deployment %s deletion failed, error: %s %s', name,
                        e.status, e.reason)
            return

        if wait_for_deletion:
            self.k8s_namespace.wait_for_deployment_deleted(name)
        logger.debug('Deployment %s deleted', name)

    def _delete_service(self, name, wait_for_deletion=True):
        logger.info('Deleting service %s', name)
        try:
            self.k8s_namespace.delete_service(name)
        except k8s.ApiException as e:
            logger.info('Service %s deletion failed, error: %s %s', name,
                        e.status, e.reason)
            return

        if wait_for_deletion:
            self.k8s_namespace.wait_for_service_deleted(name)
        logger.debug('Service %s deleted', name)

    def _delete_service_account(self, name, wait_for_deletion=True):
        logger.info('Deleting service account %s', name)
        try:
            self.k8s_namespace.delete_service_account(name)
        except k8s.ApiException as e:
            logger.info('Service account %s deletion failed, error: %s %s',
                        name, e.status, e.reason)
            return

        if wait_for_deletion:
            self.k8s_namespace.wait_for_service_account_deleted(name)
        logger.debug('Service account %s deleted', name)

    def delete_namespace(self, wait_for_deletion=True):
        logger.info('Deleting namespace %s', self.k8s_namespace.name)
        try:
            self.k8s_namespace.delete()
        except k8s.ApiException as e:
            logger.info('Namespace %s deletion failed, error: %s %s',
                        self.k8s_namespace.name, e.status, e.reason)
            return

        if wait_for_deletion:
            self.k8s_namespace.wait_for_namespace_deleted()
        logger.debug('Namespace %s deleted', self.k8s_namespace.name)

    def _wait_deployment_with_available_replicas(self, name, count=1, **kwargs):
        logger.info('Waiting for deployment %s to have %s available replica(s)',
                    name, count)
        self.k8s_namespace.wait_for_deployment_available_replicas(
            name, count, **kwargs)
        deployment = self.k8s_namespace.get_deployment(name)
        logger.info('Deployment %s has %i replicas available',
                    deployment.metadata.name,
                    deployment.status.available_replicas)

    def _wait_pod_started(self, name, **kwargs):
        logger.info('Waiting for pod %s to start', name)
        self.k8s_namespace.wait_for_pod_started(name, **kwargs)
        pod = self.k8s_namespace.get_pod(name)
        logger.info('Pod %s ready, IP: %s', pod.metadata.name,
                    pod.status.pod_ip)

    def _wait_service_neg(self, name, service_port, **kwargs):
        logger.info('Waiting for NEG for service %s', name)
        self.k8s_namespace.wait_for_service_neg(name, **kwargs)
        neg_name, neg_zones = self.k8s_namespace.get_service_neg(
            name, service_port)
        logger.info("Service %s: detected NEG=%s in zones=%s", name, neg_name,
                    neg_zones)

    @staticmethod
    def _logs_explorer_link(*,
                            deployment_name: str,
                            namespace_name: str,
                            gcp_project: str,
                            gcp_ui_url: str,
                            end_delta: Optional[timedelta] = None) -> None:
        """Output the link to test server/client logs in GCP Logs Explorer."""
        if end_delta is None:
            end_delta = timedelta(hours=1)

        time_now = _helper_datetime.iso8601_utc_time()
        time_end = _helper_datetime.iso8601_utc_time(end_delta)
        query = _logs_explorer_query({
            'resource.type': 'k8s_container',
            'resource.labels.project_id': gcp_project,
            'resource.labels.container_name': deployment_name,
            'resource.labels.namespace_name': namespace_name,
        })
        req = _logs_explorer_request({
            'query': query,
            'timeRange': f'{time_now}/{time_end}',
        })

        link = f'https://{gcp_ui_url}/logs/query;{req}?project={gcp_project}'
        # A whitespace at the end to indicate the end of the url.
        logger.info("GCP Logs Explorer link to %s:\n%s ", deployment_name, link)

    @staticmethod
    def _make_namespace_name(resource_prefix: str, resource_suffix: str,
                             name: str) -> str:
        """A helper to make consistent test app kubernetes namespace name
        for given resource prefix and suffix."""
        parts = [resource_prefix, name]
        # Avoid trailing dash when the suffix is empty.
        if resource_suffix:
            parts.append(resource_suffix)
        return '-'.join(parts)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common code used throughout tests of gRPC."""

import collections
from concurrent import futures
import threading

import grpc
import six

INVOCATION_INITIAL_METADATA = (
    ('0', 'abc'),
    ('1', 'def'),
    ('2', 'ghi'),
)
SERVICE_INITIAL_METADATA = (
    ('3', 'jkl'),
    ('4', 'mno'),
    ('5', 'pqr'),
)
SERVICE_TERMINAL_METADATA = (
    ('6', 'stu'),
    ('7', 'vwx'),
    ('8', 'yza'),
)
DETAILS = 'test details'


def metadata_transmitted(original_metadata, transmitted_metadata):
    """Judges whether or not metadata was acceptably transmitted.

  gRPC is allowed to insert key-value pairs into the metadata values given by
  applications and to reorder key-value pairs with different keys but it is not
  allowed to alter existing key-value pairs or to reorder key-value pairs with
  the same key.

  Args:
    original_metadata: A metadata value used in a test of gRPC. An iterable over
      iterables of length 2.
    transmitted_metadata: A metadata value corresponding to original_metadata
      after having been transmitted via gRPC. An iterable over iterables of
      length 2.

  Returns:
     A boolean indicating whether transmitted_metadata accurately reflects
      original_metadata after having been transmitted via gRPC.
  """
    original = collections.defaultdict(list)
    for key, value in original_metadata:
        original[key].append(value)
    transmitted = collections.defaultdict(list)
    for key, value in transmitted_metadata:
        transmitted[key].append(value)

    for key, values in six.iteritems(original):
        transmitted_values = transmitted[key]
        transmitted_iterator = iter(transmitted_values)
        try:
            for value in values:
                while True:
                    transmitted_value = next(transmitted_iterator)
                    if value == transmitted_value:
                        break
        except StopIteration:
            return False
    else:
        return True


def test_secure_channel(target, channel_credentials, server_host_override):
    """Creates an insecure Channel to a remote host.

  Args:
    host: The name of the remote host to which to connect.
    port: The port of the remote host to which to connect.
    channel_credentials: The implementations.ChannelCredentials with which to
      connect.
    server_host_override: The target name used for SSL host name checking.

  Returns:
    An implementations.Channel to the remote host through which RPCs may be
      conducted.
  """
    channel = grpc.secure_channel(target, channel_credentials, ((
        'grpc.ssl_target_name_override',
        server_host_override,
    ),))
    return channel


def test_server(max_workers=10, reuse_port=False):
    """Creates an insecure grpc server.

     These servers have SO_REUSEPORT disabled to prevent cross-talk.
     """
    return grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers),
                       options=(('grpc.so_reuseport', int(reuse_port)),))


class WaitGroup(object):

    def __init__(self, n=0):
        self.count = n
        self.cv = threading.Condition()

    def add(self, n):
        self.cv.acquire()
        self.count += n
        self.cv.release()

    def done(self):
        self.cv.acquire()
        self.count -= 1
        if self.count == 0:
            self.cv.notify_all()
        self.cv.release()

    def wait(self):
        self.cv.acquire()
        while self.count > 0:
            self.cv.wait()
        self.cv.release()


def running_under_gevent():
    try:
        from gevent import monkey
        import gevent.socket
    except ImportError:
        return False
    else:
        import socket
        return socket.socket is gevent.socket.socket



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generate XML and HTML test reports."""

try:
    from mako import exceptions
    from mako.runtime import Context
    from mako.template import Template
except (ImportError):
    pass  # Mako not installed but it is ok.
import datetime
import os
import string
import xml.etree.cElementTree as ET

import six


def _filter_msg(msg, output_format):
    """Filters out nonprintable and illegal characters from the message."""
    if output_format in ['XML', 'HTML']:
        if isinstance(msg, bytes):
            decoded_msg = msg.decode('UTF-8', 'ignore')
        else:
            decoded_msg = msg
        # keep whitespaces but remove formfeed and vertical tab characters
        # that make XML report unparsable.
        filtered_msg = ''.join(
            filter(lambda x: x in string.printable and x != '\f' and x != '\v',
                   decoded_msg))
        if output_format == 'HTML':
            filtered_msg = filtered_msg.replace('"', '&quot;')
        return filtered_msg
    else:
        return msg


def new_junit_xml_tree():
    return ET.ElementTree(ET.Element('testsuites'))


def render_junit_xml_report(resultset,
                            report_file,
                            suite_package='grpc',
                            suite_name='tests',
                            replace_dots=True,
                            multi_target=False):
    """Generate JUnit-like XML report."""
    if not multi_target:
        tree = new_junit_xml_tree()
        append_junit_xml_results(tree, resultset, suite_package, suite_name,
                                 '1', replace_dots)
        create_xml_report_file(tree, report_file)
    else:
        # To have each test result displayed as a separate target by the Resultstore/Sponge UI,
        # we generate a separate XML report file for each test result
        for shortname, results in six.iteritems(resultset):
            one_result = {shortname: results}
            tree = new_junit_xml_tree()
            append_junit_xml_results(tree, one_result,
                                     '%s_%s' % (suite_package, shortname),
                                     '%s_%s' % (suite_name, shortname), '1',
                                     replace_dots)
            per_suite_report_file = os.path.join(os.path.dirname(report_file),
                                                 shortname,
                                                 os.path.basename(report_file))
            create_xml_report_file(tree, per_suite_report_file)


def create_xml_report_file(tree, report_file):
    """Generate JUnit-like report file from xml tree ."""
    # env variable can be used to override the base location for the reports
    base_dir = os.getenv('GRPC_TEST_REPORT_BASE_DIR', None)
    if base_dir:
        report_file = os.path.join(base_dir, report_file)
    # ensure the report directory exists
    report_dir = os.path.dirname(os.path.abspath(report_file))
    if not os.path.exists(report_dir):
        os.makedirs(report_dir)
    tree.write(report_file, encoding='UTF-8')


def append_junit_xml_results(tree,
                             resultset,
                             suite_package,
                             suite_name,
                             id,
                             replace_dots=True):
    """Append a JUnit-like XML report tree with test results as a new suite."""
    if replace_dots:
        # ResultStore UI displays test suite names containing dots only as the component
        # after the last dot, which results bad info being displayed in the UI.
        # We replace dots by another character to avoid this problem.
        suite_name = suite_name.replace('.', '_')
    testsuite = ET.SubElement(tree.getroot(),
                              'testsuite',
                              id=id,
                              package=suite_package,
                              name=suite_name,
                              timestamp=datetime.datetime.now().isoformat())
    failure_count = 0
    error_count = 0
    for shortname, results in six.iteritems(resultset):
        for result in results:
            xml_test = ET.SubElement(testsuite, 'testcase', name=shortname)
            if result.elapsed_time:
                xml_test.set('time', str(result.elapsed_time))
            filtered_msg = _filter_msg(result.message, 'XML')
            if result.state == 'FAILED':
                ET.SubElement(xml_test, 'failure',
                              message='Failure').text = filtered_msg
                failure_count += 1
            elif result.state == 'TIMEOUT':
                ET.SubElement(xml_test, 'error',
                              message='Timeout').text = filtered_msg
                error_count += 1
            elif result.state == 'SKIPPED':
                ET.SubElement(xml_test, 'skipped', message='Skipped')
    testsuite.set('failures', str(failure_count))
    testsuite.set('errors', str(error_count))


def render_interop_html_report(client_langs, server_langs, test_cases,
                               auth_test_cases, http2_cases, http2_server_cases,
                               resultset, num_failures, cloud_to_prod,
                               prod_servers, http2_interop):
    """Generate HTML report for interop tests."""
    template_file = 'tools/run_tests/interop/interop_html_report.template'
    try:
        mytemplate = Template(filename=template_file, format_exceptions=True)
    except NameError:
        print(
            'Mako template is not installed. Skipping HTML report generation.')
        return
    except IOError as e:
        print(('Failed to find the template %s: %s' % (template_file, e)))
        return

    sorted_test_cases = sorted(test_cases)
    sorted_auth_test_cases = sorted(auth_test_cases)
    sorted_http2_cases = sorted(http2_cases)
    sorted_http2_server_cases = sorted(http2_server_cases)
    sorted_client_langs = sorted(client_langs)
    sorted_server_langs = sorted(server_langs)
    sorted_prod_servers = sorted(prod_servers)

    args = {
        'client_langs': sorted_client_langs,
        'server_langs': sorted_server_langs,
        'test_cases': sorted_test_cases,
        'auth_test_cases': sorted_auth_test_cases,
        'http2_cases': sorted_http2_cases,
        'http2_server_cases': sorted_http2_server_cases,
        'resultset': resultset,
        'num_failures': num_failures,
        'cloud_to_prod': cloud_to_prod,
        'prod_servers': sorted_prod_servers,
        'http2_interop': http2_interop
    }

    html_report_out_dir = 'reports'
    if not os.path.exists(html_report_out_dir):
        os.mkdir(html_report_out_dir)
    html_file_path = os.path.join(html_report_out_dir, 'index.html')
    try:
        with open(html_file_path, 'w') as output_file:
            mytemplate.render_context(Context(output_file, **args))
    except:
        print((exceptions.text_error_template().render()))
        raise


def render_perf_profiling_results(output_filepath, profile_names):
    with open(output_filepath, 'w') as output_file:
        output_file.write('<ul>\n')
        for name in profile_names:
            output_file.write('<li><a href=%s>%s</a></li>\n' % (name, name))
        output_file.write('</ul>\n')



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC helloworld.Greeter server."""

from concurrent import futures
import logging

import grpc
import helloworld_pb2
import helloworld_pb2_grpc
from request_header_validator_interceptor import \
    RequestHeaderValidatorInterceptor


class Greeter(helloworld_pb2_grpc.GreeterServicer):

    def SayHello(self, request, context):
        return helloworld_pb2.HelloReply(message='Hello, %s!' % request.name)


def serve():
    header_validator = RequestHeaderValidatorInterceptor(
        'one-time-password', '42', grpc.StatusCode.UNAUTHENTICATED,
        'Access denied!')
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10),
                         interceptors=(header_validator,))
    helloworld_pb2_grpc.add_GreeterServicer_to_server(Greeter(), server)
    server.add_insecure_port('[::]:50051')
    server.start()
    server.wait_for_termination()


if __name__ == '__main__':
    logging.basicConfig()
    serve()



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC interoperability test client."""

import argparse
import logging
import sys

from tests.fork import methods


def _args():

    def parse_bool(value):
        if value == 'true':
            return True
        if value == 'false':
            return False
        raise argparse.ArgumentTypeError('Only true/false allowed')

    parser = argparse.ArgumentParser()
    parser.add_argument('--server_host',
                        default="localhost",
                        type=str,
                        help='the host to which to connect')
    parser.add_argument('--server_port',
                        type=int,
                        required=True,
                        help='the port to which to connect')
    parser.add_argument('--test_case',
                        default='large_unary',
                        type=str,
                        help='the test case to execute')
    parser.add_argument('--use_tls',
                        default=False,
                        type=parse_bool,
                        help='require a secure connection')
    return parser.parse_args()


def _test_case_from_arg(test_case_arg):
    for test_case in methods.TestCase:
        if test_case_arg == test_case.value:
            return test_case
    else:
        raise ValueError('No test case "%s"!' % test_case_arg)


def test_fork():
    logging.basicConfig(level=logging.INFO)
    args = vars(_args())
    if args['test_case'] == "all":
        for test_case in methods.TestCase:
            test_case.run_test(args)
    else:
        test_case = _test_case_from_arg(args['test_case'])
        test_case.run_test(args)


if __name__ == '__main__':
    test_fork()



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import unittest

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_REQUEST = b''
_RESPONSE = b''

_UNARY_UNARY = '/test/UnaryUnary'
_UNARY_STREAM = '/test/UnaryStream'
_STREAM_UNARY = '/test/StreamUnary'
_STREAM_STREAM = '/test/StreamStream'


def handle_unary_unary(request, servicer_context):
    return _RESPONSE


def handle_unary_stream(request, servicer_context):
    for _ in range(test_constants.STREAM_LENGTH):
        yield _RESPONSE


def handle_stream_unary(request_iterator, servicer_context):
    for request in request_iterator:
        pass
    return _RESPONSE


def handle_stream_stream(request_iterator, servicer_context):
    for request in request_iterator:
        yield _RESPONSE


class _MethodHandler(grpc.RpcMethodHandler):

    def __init__(self, request_streaming, response_streaming):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            self.stream_stream = handle_stream_stream
        elif self.request_streaming:
            self.stream_unary = handle_stream_unary
        elif self.response_streaming:
            self.unary_stream = handle_unary_stream
        else:
            self.unary_unary = handle_unary_unary


class _GenericHandler(grpc.GenericRpcHandler):

    def service(self, handler_call_details):
        if handler_call_details.method == _UNARY_UNARY:
            return _MethodHandler(False, False)
        elif handler_call_details.method == _UNARY_STREAM:
            return _MethodHandler(False, True)
        elif handler_call_details.method == _STREAM_UNARY:
            return _MethodHandler(True, False)
        elif handler_call_details.method == _STREAM_STREAM:
            return _MethodHandler(True, True)
        else:
            return None


class EmptyMessageTest(unittest.TestCase):

    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_generic_rpc_handlers((_GenericHandler(),))
        port = self._server.add_insecure_port('[::]:0')
        self._server.start()
        self._channel = grpc.insecure_channel('localhost:%d' % port)

    def tearDown(self):
        self._server.stop(0)
        self._channel.close()

    def testUnaryUnary(self):
        response = self._channel.unary_unary(_UNARY_UNARY)(_REQUEST)
        self.assertEqual(_RESPONSE, response)

    def testUnaryStream(self):
        response_iterator = self._channel.unary_stream(_UNARY_STREAM)(_REQUEST)
        self.assertSequenceEqual([_RESPONSE] * test_constants.STREAM_LENGTH,
                                 list(response_iterator))

    def testStreamUnary(self):
        response = self._channel.stream_unary(_STREAM_UNARY)(iter(
            [_REQUEST] * test_constants.STREAM_LENGTH))
        self.assertEqual(_RESPONSE, response)

    def testStreamStream(self):
        response_iterator = self._channel.stream_stream(_STREAM_STREAM)(iter(
            [_REQUEST] * test_constants.STREAM_LENGTH))
        self.assertSequenceEqual([_RESPONSE] * test_constants.STREAM_LENGTH,
                                 list(response_iterator))


if __name__ == '__main__':
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This contains helpers for gRPC services defined in
https://github.com/grpc/grpc-proto/blob/master/grpc/channelz/v1/channelz.proto
"""
import ipaddress
import logging
from typing import Iterator, Optional

import grpc
from grpc_channelz.v1 import channelz_pb2
from grpc_channelz.v1 import channelz_pb2_grpc

import framework.rpc

logger = logging.getLogger(__name__)

# Type aliases
# Channel
Channel = channelz_pb2.Channel
ChannelConnectivityState = channelz_pb2.ChannelConnectivityState
ChannelState = ChannelConnectivityState.State  # pylint: disable=no-member
_GetTopChannelsRequest = channelz_pb2.GetTopChannelsRequest
_GetTopChannelsResponse = channelz_pb2.GetTopChannelsResponse
# Subchannel
Subchannel = channelz_pb2.Subchannel
_GetSubchannelRequest = channelz_pb2.GetSubchannelRequest
_GetSubchannelResponse = channelz_pb2.GetSubchannelResponse
# Server
Server = channelz_pb2.Server
_GetServersRequest = channelz_pb2.GetServersRequest
_GetServersResponse = channelz_pb2.GetServersResponse
# Sockets
Socket = channelz_pb2.Socket
SocketRef = channelz_pb2.SocketRef
_GetSocketRequest = channelz_pb2.GetSocketRequest
_GetSocketResponse = channelz_pb2.GetSocketResponse
Address = channelz_pb2.Address
Security = channelz_pb2.Security
# Server Sockets
_GetServerSocketsRequest = channelz_pb2.GetServerSocketsRequest
_GetServerSocketsResponse = channelz_pb2.GetServerSocketsResponse


class ChannelzServiceClient(framework.rpc.grpc.GrpcClientHelper):
    stub: channelz_pb2_grpc.ChannelzStub

    def __init__(self, channel: grpc.Channel):
        super().__init__(channel, channelz_pb2_grpc.ChannelzStub)

    @staticmethod
    def is_sock_tcpip_address(address: Address):
        return address.WhichOneof('address') == 'tcpip_address'

    @staticmethod
    def is_ipv4(tcpip_address: Address.TcpIpAddress):
        # According to proto, tcpip_address.ip_address is either IPv4 or IPv6.
        # Correspondingly, it's either 4 bytes or 16 bytes in length.
        return len(tcpip_address.ip_address) == 4

    @classmethod
    def sock_address_to_str(cls, address: Address):
        if cls.is_sock_tcpip_address(address):
            tcpip_address: Address.TcpIpAddress = address.tcpip_address
            if cls.is_ipv4(tcpip_address):
                ip = ipaddress.IPv4Address(tcpip_address.ip_address)
            else:
                ip = ipaddress.IPv6Address(tcpip_address.ip_address)
            return f'{ip}:{tcpip_address.port}'
        else:
            raise NotImplementedError('Only tcpip_address implemented')

    @classmethod
    def sock_addresses_pretty(cls, socket: Socket):
        return (f'local={cls.sock_address_to_str(socket.local)}, '
                f'remote={cls.sock_address_to_str(socket.remote)}')

    @staticmethod
    def find_server_socket_matching_client(server_sockets: Iterator[Socket],
                                           client_socket: Socket) -> Socket:
        for server_socket in server_sockets:
            if server_socket.remote == client_socket.local:
                return server_socket
        return None

    def find_channels_for_target(self, target: str,
                                 **kwargs) -> Iterator[Channel]:
        return (channel for channel in self.list_channels(**kwargs)
                if channel.data.target == target)

    def find_server_listening_on_port(self, port: int,
                                      **kwargs) -> Optional[Server]:
        for server in self.list_servers(**kwargs):
            listen_socket_ref: SocketRef
            for listen_socket_ref in server.listen_socket:
                listen_socket = self.get_socket(listen_socket_ref.socket_id,
                                                **kwargs)
                listen_address: Address = listen_socket.local
                if (self.is_sock_tcpip_address(listen_address) and
                        listen_address.tcpip_address.port == port):
                    return server
        return None

    def list_channels(self, **kwargs) -> Iterator[Channel]:
        """
        Iterate over all pages of all root channels.

        Root channels are those which application has directly created.
        This does not include subchannels nor non-top level channels.
        """
        start: int = -1
        response: Optional[_GetTopChannelsResponse] = None
        while start < 0 or not response.end:
            # From proto: To request subsequent pages, the client generates this
            # value by adding 1 to the highest seen result ID.
            start += 1
            response = self.call_unary_with_deadline(
                rpc='GetTopChannels',
                req=_GetTopChannelsRequest(start_channel_id=start),
                **kwargs)
            for channel in response.channel:
                start = max(start, channel.ref.channel_id)
                yield channel

    def list_servers(self, **kwargs) -> Iterator[Server]:
        """Iterate over all pages of all servers that exist in the process."""
        start: int = -1
        response: Optional[_GetServersResponse] = None
        while start < 0 or not response.end:
            # From proto: To request subsequent pages, the client generates this
            # value by adding 1 to the highest seen result ID.
            start += 1
            response = self.call_unary_with_deadline(
                rpc='GetServers',
                req=_GetServersRequest(start_server_id=start),
                **kwargs)
            for server in response.server:
                start = max(start, server.ref.server_id)
                yield server

    def list_server_sockets(self, server: Server, **kwargs) -> Iterator[Socket]:
        """List all server sockets that exist in server process.

        Iterating over the results will resolve additional pages automatically.
        """
        start: int = -1
        response: Optional[_GetServerSocketsResponse] = None
        while start < 0 or not response.end:
            # From proto: To request subsequent pages, the client generates this
            # value by adding 1 to the highest seen result ID.
            start += 1
            response = self.call_unary_with_deadline(
                rpc='GetServerSockets',
                req=_GetServerSocketsRequest(server_id=server.ref.server_id,
                                             start_socket_id=start),
                **kwargs)
            socket_ref: SocketRef
            for socket_ref in response.socket_ref:
                start = max(start, socket_ref.socket_id)
                # Yield actual socket
                yield self.get_socket(socket_ref.socket_id, **kwargs)

    def list_channel_sockets(self, channel: Channel,
                             **kwargs) -> Iterator[Socket]:
        """List all sockets of all subchannels of a given channel."""
        for subchannel in self.list_channel_subchannels(channel, **kwargs):
            yield from self.list_subchannels_sockets(subchannel, **kwargs)

    def list_channel_subchannels(self, channel: Channel,
                                 **kwargs) -> Iterator[Subchannel]:
        """List all subchannels of a given channel."""
        for subchannel_ref in channel.subchannel_ref:
            yield self.get_subchannel(subchannel_ref.subchannel_id, **kwargs)

    def list_subchannels_sockets(self, subchannel: Subchannel,
                                 **kwargs) -> Iterator[Socket]:
        """List all sockets of a given subchannel."""
        for socket_ref in subchannel.socket_ref:
            yield self.get_socket(socket_ref.socket_id, **kwargs)

    def get_subchannel(self, subchannel_id, **kwargs) -> Subchannel:
        """Return a single Subchannel, otherwise raises RpcError."""
        response: _GetSubchannelResponse = self.call_unary_with_deadline(
            rpc='GetSubchannel',
            req=_GetSubchannelRequest(subchannel_id=subchannel_id),
            **kwargs)
        return response.subchannel

    def get_socket(self, socket_id, **kwargs) -> Socket:
        """Return a single Socket, otherwise raises RpcError."""
        response: _GetSocketResponse = self.call_unary_with_deadline(
            rpc='GetSocket',
            req=_GetSocketRequest(socket_id=socket_id),
            **kwargs)
        return response.socket



#!/usr/bin/env python3
#
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Computes the diff between two qps runs and outputs significant results """

import argparse
import json
import multiprocessing
import os
import shutil
import subprocess
import sys

import qps_scenarios
import tabulate

sys.path.append(
    os.path.join(os.path.dirname(sys.argv[0]), '..', 'microbenchmarks',
                 'bm_diff'))
import bm_speedup

sys.path.append(
    os.path.join(os.path.dirname(sys.argv[0]), '..', '..', 'run_tests',
                 'python_utils'))
import check_on_pr


def _args():
    argp = argparse.ArgumentParser(description='Perform diff on QPS Driver')
    argp.add_argument('-d',
                      '--diff_base',
                      type=str,
                      help='Commit or branch to compare the current one to')
    argp.add_argument(
        '-l',
        '--loops',
        type=int,
        default=4,
        help='Number of loops for each benchmark. More loops cuts down on noise'
    )
    argp.add_argument('-j',
                      '--jobs',
                      type=int,
                      default=multiprocessing.cpu_count(),
                      help='Number of CPUs to use')
    args = argp.parse_args()
    assert args.diff_base, "diff_base must be set"
    return args


def _make_cmd(jobs):
    return ['make', '-j', '%d' % jobs, 'qps_json_driver', 'qps_worker']


def build(name, jobs):
    shutil.rmtree('qps_diff_%s' % name, ignore_errors=True)
    subprocess.check_call(['git', 'submodule', 'update'])
    try:
        subprocess.check_call(_make_cmd(jobs))
    except subprocess.CalledProcessError as e:
        subprocess.check_call(['make', 'clean'])
        subprocess.check_call(_make_cmd(jobs))
    os.rename('bins', 'qps_diff_%s' % name)


def _run_cmd(name, scenario, fname):
    return [
        'qps_diff_%s/opt/qps_json_driver' % name, '--scenarios_json', scenario,
        '--json_file_out', fname
    ]


def run(name, scenarios, loops):
    for sn in scenarios:
        for i in range(0, loops):
            fname = "%s.%s.%d.json" % (sn, name, i)
            subprocess.check_call(_run_cmd(name, scenarios[sn], fname))


def _load_qps(fname):
    try:
        with open(fname) as f:
            return json.loads(f.read())['qps']
    except IOError as e:
        print(("IOError occurred reading file: %s" % fname))
        return None
    except ValueError as e:
        print(("ValueError occurred reading file: %s" % fname))
        return None


def _median(ary):
    assert (len(ary))
    ary = sorted(ary)
    n = len(ary)
    if n % 2 == 0:
        return (ary[(n - 1) / 2] + ary[(n - 1) / 2 + 1]) / 2.0
    else:
        return ary[n / 2]


def diff(scenarios, loops, old, new):
    old_data = {}
    new_data = {}

    # collect data
    for sn in scenarios:
        old_data[sn] = []
        new_data[sn] = []
        for i in range(loops):
            old_data[sn].append(_load_qps("%s.%s.%d.json" % (sn, old, i)))
            new_data[sn].append(_load_qps("%s.%s.%d.json" % (sn, new, i)))

    # crunch data
    headers = ['Benchmark', 'qps']
    rows = []
    for sn in scenarios:
        mdn_diff = abs(_median(new_data[sn]) - _median(old_data[sn]))
        print(('%s: %s=%r %s=%r mdn_diff=%r' %
               (sn, new, new_data[sn], old, old_data[sn], mdn_diff)))
        s = bm_speedup.speedup(new_data[sn], old_data[sn], 10e-5)
        if abs(s) > 3 and mdn_diff > 0.5:
            rows.append([sn, '%+d%%' % s])

    if rows:
        return tabulate.tabulate(rows, headers=headers, floatfmt='+.2f')
    else:
        return None


def main(args):
    build('new', args.jobs)

    if args.diff_base:
        where_am_i = subprocess.check_output(
            ['git', 'rev-parse', '--abbrev-ref', 'HEAD']).decode().strip()
        subprocess.check_call(['git', 'checkout', args.diff_base])
        try:
            build('old', args.jobs)
        finally:
            subprocess.check_call(['git', 'checkout', where_am_i])
            subprocess.check_call(['git', 'submodule', 'update'])

    run('new', qps_scenarios._SCENARIOS, args.loops)
    run('old', qps_scenarios._SCENARIOS, args.loops)

    diff_output = diff(qps_scenarios._SCENARIOS, args.loops, 'old', 'new')

    if diff_output:
        text = '[qps] Performance differences noted:\n%s' % diff_output
    else:
        text = '[qps] No significant performance differences'
    print(('%s' % text))
    check_on_pr.check_on_pr('QPS', '```\n%s\n```' % text)


if __name__ == '__main__':
    args = _args()
    main(args)



#!/usr/bin/env python3
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Utilities for manipulating JSON data that represents microbenchmark results.

import os

# template arguments and dynamic arguments of individual benchmark types
# Example benchmark name: "BM_UnaryPingPong<TCP, NoOpMutator, NoOpMutator>/0/0"
_BM_SPECS = {
    'BM_UnaryPingPong': {
        'tpl': ['fixture', 'client_mutator', 'server_mutator'],
        'dyn': ['request_size', 'response_size'],
    },
    'BM_PumpStreamClientToServer': {
        'tpl': ['fixture'],
        'dyn': ['request_size'],
    },
    'BM_PumpStreamServerToClient': {
        'tpl': ['fixture'],
        'dyn': ['request_size'],
    },
    'BM_StreamingPingPong': {
        'tpl': ['fixture', 'client_mutator', 'server_mutator'],
        'dyn': ['request_size', 'request_count'],
    },
    'BM_StreamingPingPongMsgs': {
        'tpl': ['fixture', 'client_mutator', 'server_mutator'],
        'dyn': ['request_size'],
    },
    'BM_PumpStreamServerToClient_Trickle': {
        'tpl': [],
        'dyn': ['request_size', 'bandwidth_kilobits'],
    },
    'BM_PumpUnbalancedUnary_Trickle': {
        'tpl': [],
        'dyn': ['cli_req_size', 'svr_req_size', 'bandwidth_kilobits'],
    },
    'BM_ErrorStringOnNewError': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_ErrorStringRepeatedly': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_ErrorGetStatus': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_ErrorGetStatusCode': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_ErrorHttpError': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_HasClearGrpcStatus': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_IsolatedFilter': {
        'tpl': ['fixture', 'client_mutator'],
        'dyn': [],
    },
    'BM_HpackEncoderEncodeHeader': {
        'tpl': ['fixture'],
        'dyn': ['end_of_stream', 'request_size'],
    },
    'BM_HpackParserParseHeader': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_CallCreateDestroy': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_Zalloc': {
        'tpl': [],
        'dyn': ['request_size'],
    },
    'BM_PollEmptyPollset_SpeedOfLight': {
        'tpl': [],
        'dyn': ['request_size', 'request_count'],
    },
    'BM_StreamCreateSendInitialMetadataDestroy': {
        'tpl': ['fixture'],
        'dyn': [],
    },
    'BM_TransportStreamSend': {
        'tpl': [],
        'dyn': ['request_size'],
    },
    'BM_TransportStreamRecv': {
        'tpl': [],
        'dyn': ['request_size'],
    },
    'BM_StreamingPingPongWithCoalescingApi': {
        'tpl': ['fixture', 'client_mutator', 'server_mutator'],
        'dyn': ['request_size', 'request_count', 'end_of_stream'],
    },
    'BM_Base16SomeStuff': {
        'tpl': [],
        'dyn': ['request_size'],
    }
}


def numericalize(s):
    """Convert abbreviations like '100M' or '10k' to a number."""
    if not s:
        return ''
    if s[-1] == 'k':
        return float(s[:-1]) * 1024
    if s[-1] == 'M':
        return float(s[:-1]) * 1024 * 1024
    if 0 <= (ord(s[-1]) - ord('0')) <= 9:
        return float(s)
    assert 'not a number: %s' % s


def parse_name(name):
    cpp_name = name
    if '<' not in name and '/' not in name and name not in _BM_SPECS:
        return {'name': name, 'cpp_name': name}
    rest = name
    out = {}
    tpl_args = []
    dyn_args = []
    if '<' in rest:
        tpl_bit = rest[rest.find('<') + 1:rest.rfind('>')]
        arg = ''
        nesting = 0
        for c in tpl_bit:
            if c == '<':
                nesting += 1
                arg += c
            elif c == '>':
                nesting -= 1
                arg += c
            elif c == ',':
                if nesting == 0:
                    tpl_args.append(arg.strip())
                    arg = ''
                else:
                    arg += c
            else:
                arg += c
        tpl_args.append(arg.strip())
        rest = rest[:rest.find('<')] + rest[rest.rfind('>') + 1:]
    if '/' in rest:
        s = rest.split('/')
        rest = s[0]
        dyn_args = s[1:]
    name = rest
    assert name in _BM_SPECS, '_BM_SPECS needs to be expanded for %s' % name
    assert len(dyn_args) == len(_BM_SPECS[name]['dyn'])
    assert len(tpl_args) == len(_BM_SPECS[name]['tpl'])
    out['name'] = name
    out['cpp_name'] = cpp_name
    out.update(
        dict((k, numericalize(v))
             for k, v in zip(_BM_SPECS[name]['dyn'], dyn_args)))
    out.update(dict(zip(_BM_SPECS[name]['tpl'], tpl_args)))
    return out


def expand_json(js, js2=None):
    if not js and not js2:
        raise StopIteration()
    if not js:
        js = js2
    for bm in js['benchmarks']:
        if bm['name'].endswith('_stddev') or bm['name'].endswith('_mean'):
            continue
        context = js['context']
        if 'label' in bm:
            labels_list = [
                s.split(':')
                for s in bm['label'].strip().split(' ')
                if len(s) and s[0] != '#'
            ]
            for el in labels_list:
                el[0] = el[0].replace('/iter', '_per_iteration')
            labels = dict(labels_list)
        else:
            labels = {}
        # TODO(jtattermusch): grabbing kokoro env values shouldn't be buried
        # deep in the JSON conversion logic.
        # Link the data to a kokoro job run by adding
        # well known kokoro env variables as metadata for each row
        row = {
            'jenkins_build': os.environ.get('KOKORO_BUILD_NUMBER', ''),
            'jenkins_job': os.environ.get('KOKORO_JOB_NAME', ''),
        }
        row.update(context)
        row.update(bm)
        row.update(parse_name(row['name']))
        row.update(labels)
        # TODO(jtattermusch): add a comment explaining what's the point
        # of merging values of some of the columns js2 into the row.
        # Empirically, the js contains data from "counters" config
        # and js2 contains data from the "opt" config, but the point of merging
        # really deserves further explanation.
        if js2:
            for bm2 in js2['benchmarks']:
                if bm['name'] == bm2['name'] and 'already_used' not in bm2:
                    row['cpu_time'] = bm2['cpu_time']
                    row['real_time'] = bm2['real_time']
                    row['iterations'] = bm2['iterations']
                    bm2['already_used'] = True
                    break
        yield row



# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import dataclasses
import datetime
import functools
import logging
from typing import Any, Dict, FrozenSet, Optional

from framework.helpers import retryers
from framework.infrastructure import gcp

logger = logging.getLogger(__name__)

# Type aliases
_timedelta = datetime.timedelta
_HttpRequest = gcp.api.HttpRequest


class EtagConflict(gcp.api.Error):
    """
    Indicates concurrent policy changes.

    https://cloud.google.com/iam/docs/policies#etag
    """


def handle_etag_conflict(func):

    def wrap_retry_on_etag_conflict(*args, **kwargs):
        retryer = retryers.exponential_retryer_with_timeout(
            retry_on_exceptions=(EtagConflict, gcp.api.TransportError),
            wait_min=_timedelta(seconds=1),
            wait_max=_timedelta(seconds=10),
            timeout=_timedelta(minutes=2))
        return retryer(func, *args, **kwargs)

    return wrap_retry_on_etag_conflict


def _replace_binding(policy: 'Policy', binding: 'Policy.Binding',
                     new_binding: 'Policy.Binding') -> 'Policy':
    new_bindings = set(policy.bindings)
    new_bindings.discard(binding)
    new_bindings.add(new_binding)
    return dataclasses.replace(policy, bindings=frozenset(new_bindings))  # pylint: disable=too-many-function-args


@dataclasses.dataclass(frozen=True)
class ServiceAccount:
    """An IAM service account.

    https://cloud.google.com/iam/docs/reference/rest/v1/projects.serviceAccounts
    Note: "etag" field is skipped because it's deprecated
    """
    name: str
    projectId: str
    uniqueId: str
    email: str
    oauth2ClientId: str
    displayName: str = ''
    description: str = ''
    disabled: bool = False

    @classmethod
    def from_response(cls, response: Dict[str, Any]) -> 'ServiceAccount':
        return cls(name=response['name'],
                   projectId=response['projectId'],
                   uniqueId=response['uniqueId'],
                   email=response['email'],
                   oauth2ClientId=response['oauth2ClientId'],
                   description=response.get('description', ''),
                   displayName=response.get('displayName', ''),
                   disabled=response.get('disabled', False))

    def as_dict(self) -> Dict[str, Any]:
        return dataclasses.asdict(self)


@dataclasses.dataclass(frozen=True)
class Expr:
    """
    Represents a textual expression in the Common Expression Language syntax.

    https://cloud.google.com/iam/docs/reference/rest/v1/Expr
    """
    expression: str
    title: str = ''
    description: str = ''
    location: str = ''

    @classmethod
    def from_response(cls, response: Dict[str, Any]) -> 'Expr':
        return cls(**response)

    def as_dict(self) -> Dict[str, Any]:
        return dataclasses.asdict(self)


@dataclasses.dataclass(frozen=True)
class Policy:
    """An Identity and Access Management (IAM) policy, which specifies
    access controls for Google Cloud resources.

    https://cloud.google.com/iam/docs/reference/rest/v1/Policy
    Note: auditConfigs not supported by this implementation.
    """

    @dataclasses.dataclass(frozen=True)
    class Binding:
        """Policy Binding. Associates members with a role.

        https://cloud.google.com/iam/docs/reference/rest/v1/Policy#binding
        """
        role: str
        members: FrozenSet[str]
        condition: Optional[Expr] = None

        @classmethod
        def from_response(cls, response: Dict[str, Any]) -> 'Policy.Binding':
            fields = {
                'role': response['role'],
                'members': frozenset(response.get('members', [])),
            }
            if 'condition' in response:
                fields['condition'] = Expr.from_response(response['condition'])

            return cls(**fields)

        def as_dict(self) -> Dict[str, Any]:
            result = {
                'role': self.role,
                'members': list(self.members),
            }
            if self.condition is not None:
                result['condition'] = self.condition.as_dict()
            return result

    bindings: FrozenSet[Binding]
    etag: str
    version: Optional[int] = None

    @functools.lru_cache(maxsize=128)
    def find_binding_for_role(
            self,
            role: str,
            condition: Optional[Expr] = None) -> Optional['Policy.Binding']:
        results = (binding for binding in self.bindings
                   if binding.role == role and binding.condition == condition)
        return next(results, None)

    @classmethod
    def from_response(cls, response: Dict[str, Any]) -> 'Policy':
        bindings = frozenset(
            cls.Binding.from_response(b) for b in response.get('bindings', []))
        return cls(bindings=bindings,
                   etag=response['etag'],
                   version=response.get('version'))

    def as_dict(self) -> Dict[str, Any]:
        result = {
            'bindings': [binding.as_dict() for binding in self.bindings],
            'etag': self.etag,
        }
        if self.version is not None:
            result['version'] = self.version
        return result


class IamV1(gcp.api.GcpProjectApiResource):
    """
    Identity and Access Management (IAM) API.

    https://cloud.google.com/iam/docs/reference/rest
    """
    _service_accounts: gcp.api.discovery.Resource

    # Operations that affect conditional role bindings must specify version 3.
    # Otherwise conditions are omitted, and role names returned with a suffix,
    # f.e. roles/iam.workloadIdentityUser_withcond_f1ec33c9beb41857dbf0
    # https://cloud.google.com/iam/docs/reference/rest/v1/Policy#FIELDS.version
    POLICY_VERSION: int = 3

    def __init__(self, api_manager: gcp.api.GcpApiManager, project: str):
        super().__init__(api_manager.iam('v1'), project)
        # Shortcut to projects/*/serviceAccounts/ endpoints
        self._service_accounts = self.api.projects().serviceAccounts()

    def service_account_resource_name(self, account) -> str:
        """
        Returns full resource name of the service account.

        The resource name of the service account in the following format:
        projects/{PROJECT_ID}/serviceAccounts/{ACCOUNT}.
        The ACCOUNT value can be the email address or the uniqueId of the
        service account.
        Ref https://cloud.google.com/iam/docs/reference/rest/v1/projects.serviceAccounts/get

        Args:
            account: The ACCOUNT value
        """
        return f'projects/{self.project}/serviceAccounts/{account}'

    def get_service_account(self, account: str) -> ServiceAccount:
        resource_name = self.service_account_resource_name(account)
        request: _HttpRequest = self._service_accounts.get(name=resource_name)
        response: Dict[str, Any] = self._execute(request)
        logger.debug('Loaded Service Account:\n%s',
                     self.resource_pretty_format(response))
        return ServiceAccount.from_response(response)

    def get_service_account_iam_policy(self, account: str) -> Policy:
        resource_name = self.service_account_resource_name(account)
        request: _HttpRequest = self._service_accounts.getIamPolicy(
            resource=resource_name,
            options_requestedPolicyVersion=self.POLICY_VERSION)
        response: Dict[str, Any] = self._execute(request)
        logger.debug('Loaded Service Account Policy:\n%s',
                     self.resource_pretty_format(response))
        return Policy.from_response(response)

    def set_service_account_iam_policy(self, account: str,
                                       policy: Policy) -> Policy:
        """Sets the IAM policy that is attached to a service account.

        https://cloud.google.com/iam/docs/reference/rest/v1/projects.serviceAccounts/setIamPolicy
        """
        resource_name = self.service_account_resource_name(account)
        body = {'policy': policy.as_dict()}
        logger.debug('Updating Service Account %s policy:\n%s', account,
                     self.resource_pretty_format(body))
        try:
            request: _HttpRequest = self._service_accounts.setIamPolicy(
                resource=resource_name, body=body)
            response: Dict[str, Any] = self._execute(request)
            return Policy.from_response(response)
        except gcp.api.ResponseError as error:
            if error.status == 409:
                # https://cloud.google.com/iam/docs/policies#etag
                logger.debug(error)
                raise EtagConflict from error
            raise

    @handle_etag_conflict
    def add_service_account_iam_policy_binding(self, account: str, role: str,
                                               member: str) -> None:
        """Add an IAM policy binding to an IAM service account.

        See for details on updating policy bindings:
        https://cloud.google.com/iam/docs/reference/rest/v1/projects.serviceAccounts/setIamPolicy
        """
        policy: Policy = self.get_service_account_iam_policy(account)
        binding: Optional[Policy.Binding] = policy.find_binding_for_role(role)
        if binding and member in binding.members:
            logger.debug('Member %s already has role %s for Service Account %s',
                         member, role, account)
            return

        if binding is None:
            updated_binding = Policy.Binding(role, frozenset([member]))
        else:
            updated_members: FrozenSet[str] = binding.members.union({member})
            updated_binding: Policy.Binding = dataclasses.replace(  # pylint: disable=too-many-function-args
                binding,
                members=updated_members)

        updated_policy: Policy = _replace_binding(policy, binding,
                                                  updated_binding)
        self.set_service_account_iam_policy(account, updated_policy)
        logger.debug('Role %s granted to member %s for Service Account %s',
                     role, member, account)

    @handle_etag_conflict
    def remove_service_account_iam_policy_binding(self, account: str, role: str,
                                                  member: str) -> None:
        """Remove an IAM policy binding from the IAM policy of a service
        account.

        See for details on updating policy bindings:
        https://cloud.google.com/iam/docs/reference/rest/v1/projects.serviceAccounts/setIamPolicy
        """
        policy: Policy = self.get_service_account_iam_policy(account)
        binding: Optional[Policy.Binding] = policy.find_binding_for_role(role)

        if binding is None:
            logger.debug('Noop: Service Account %s has no bindings for role %s',
                         account, role)
            return
        if member not in binding.members:
            logger.debug(
                'Noop: Service Account %s binding for role %s has no member %s',
                account, role, member)
            return

        updated_members: FrozenSet[str] = binding.members.difference({member})
        updated_binding: Policy.Binding = dataclasses.replace(  # pylint: disable=too-many-function-args
            binding,
            members=updated_members)
        updated_policy: Policy = _replace_binding(policy, binding,
                                                  updated_binding)
        self.set_service_account_iam_policy(account, updated_policy)
        logger.debug('Role %s revoked from member %s for Service Account %s',
                     role, member, account)



# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python AsyncIO implementation of the GRPC hellostreamingworld.MultiGreeter client."""

import asyncio
import logging

import grpc
import hellostreamingworld_pb2
import hellostreamingworld_pb2_grpc


async def run() -> None:
    async with grpc.aio.insecure_channel("localhost:50051") as channel:
        stub = hellostreamingworld_pb2_grpc.MultiGreeterStub(channel)

        # Read from an async generator
        async for response in stub.sayHello(
            hellostreamingworld_pb2.HelloRequest(name="you")):
            print("Greeter client received from async generator: " +
                  response.message)

        # Direct read from the stub
        hello_stream = stub.sayHello(
            hellostreamingworld_pb2.HelloRequest(name="you"))
        while True:
            response = await hello_stream.read()
            if response == grpc.aio.EOF:
                break
            print("Greeter client received from direct read: " +
                  response.message)


if __name__ == "__main__":
    logging.basicConfig()
    asyncio.run(run())
